# Copyright 2019 Inspur Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Share driver test for Inspur InStorage
"""

from unittest import mock

import ddt
from eventlet import greenthread
from oslo_concurrency import processutils
from oslo_config import cfg
import paramiko

from manila import context
from manila import exception
from manila.share import driver
from manila.share.drivers.inspur.instorage import cli_helper
from manila.share.drivers.inspur.instorage import instorage
from manila import test
from manila.tests import fake_share
from manila import utils as manila_utils

CONF = cfg.CONF


class FakeConfig(object):
    def __init__(self, *args, **kwargs):
        self.driver_handles_share_servers = False
        self.share_driver = 'fake_share_driver_name'
        self.share_backend_name = 'fake_instorage'
        self.instorage_nas_ip = kwargs.get(
            'instorage_nas_ip', 'some_ip')
        self.instorage_nas_port = kwargs.get(
            'instorage_nas_port', 'some_port')
        self.instorage_nas_login = kwargs.get(
            'instorage_nas_login', 'username')
        self.instorage_nas_password = kwargs.get(
            'instorage_nas_password', 'password')
        self.instorage_nas_pools = kwargs.get(
            'instorage_nas_pools', ['fakepool'])
        self.network_config_group = kwargs.get(
            "network_config_group", "fake_network_config_group")
        self.admin_network_config_group = kwargs.get(
            "admin_network_config_group", "fake_admin_network_config_group")
        self.config_group = kwargs.get("config_group", "fake_config_group")
        self.reserved_share_percentage = kwargs.get(
            "reserved_share_percentage", 0)
        self.reserved_share_from_snapshot_percentage = kwargs.get(
            "reserved_share_from_snapshot_percentage", 0)
        self.max_over_subscription_ratio = kwargs.get(
            "max_over_subscription_ratio", 0)
        self.filter_function = kwargs.get("filter_function", None)
        self.goodness_function = kwargs.get("goodness_function", None)

    def safe_get(self, key):
        return getattr(self, key)

    def append_config_values(self, *args, **kwargs):
        pass


@ddt.ddt
class InStorageShareDriverTestCase(test.TestCase):
    def __init__(self, *args, **kwargs):
        super(InStorageShareDriverTestCase, self).__init__(*args, **kwargs)
        self._ctxt = context.get_admin_context()
        self.configuration = FakeConfig()
        self.share = fake_share.fake_share()
        self.share_instance = fake_share.fake_share_instance(
            self.share, host='H@B#P'
        )

    def setUp(self):
        self.mock_object(instorage.CONF, '_check_required_opts')
        self.driver = instorage.InStorageShareDriver(
            configuration=self.configuration
        )
        super(InStorageShareDriverTestCase, self).setUp()

    def test_check_for_setup_error_failed_no_nodes(self):
        mock_gni = mock.Mock(return_value={})
        self.mock_object(
            instorage.InStorageAssistant, 'get_nodes_info', mock_gni
        )

        self.assertRaises(
            exception.ShareBackendException,
            self.driver.check_for_setup_error
        )

    def test_check_for_setup_error_failed_pool_invalid(self):
        mock_gni = mock.Mock(return_value={'node1': {}})
        self.mock_object(
            instorage.InStorageAssistant, 'get_nodes_info', mock_gni
        )
        mock_gap = mock.Mock(return_value=['pool0'])
        self.mock_object(
            instorage.InStorageAssistant, 'get_available_pools', mock_gap
        )

        self.assertRaises(
            exception.InvalidParameterValue,
            self.driver.check_for_setup_error
        )

    def test_check_for_setup_error_success(self):
        mock_gni = mock.Mock(return_value={'node1': {}})
        self.mock_object(
            instorage.InStorageAssistant, 'get_nodes_info', mock_gni
        )
        mock_gap = mock.Mock(return_value=['fakepool', 'pool0'])
        self.mock_object(
            instorage.InStorageAssistant, 'get_available_pools', mock_gap
        )

        self.driver.check_for_setup_error()
        mock_gni.assert_called_once()
        mock_gap.assert_called_once()

    def test__update_share_stats(self):
        pool_attr = {
            'pool0': {
                'pool_name': 'pool0',
                'total_capacity_gb': 110,
                'free_capacity_gb': 100,
                'allocated_capacity_gb': 10,
                'reserved_percentage': 0,
                'reserved_snapshot_percentage': 0,
                'qos': False,
                'dedupe': False,
                'compression': False,
                'thin_provisioning': False,
                'max_over_subscription_ratio': 0
            }
        }
        mock_gpa = mock.Mock(return_value=pool_attr)
        self.mock_object(
            instorage.InStorageAssistant, 'get_pools_attr', mock_gpa
        )
        mock_uss = mock.Mock()
        self.mock_object(driver.ShareDriver, '_update_share_stats', mock_uss)

        self.driver._update_share_stats()

        mock_gpa.assert_called_once_with(['fakepool'])
        stats = {
            'share_backend_name': 'fake_instorage',
            'vendor_name': 'INSPUR',
            'driver_version': '1.0.0',
            'storage_protocol': 'NFS_CIFS',
            'reserved_percentage': 0,
            'reserved_snapshot_percentage': 0,
            'max_over_subscription_ratio': 0,
            'snapshot_support': False,
            'create_share_from_snapshot_support': False,
            'revert_to_snapshot_support': False,
            'qos': False,
            'total_capacity_gb': 110,
            'free_capacity_gb': 100,
            'pools': [pool_attr['pool0']]
        }
        mock_uss.assert_called_once_with(stats)

    @ddt.data(
        {'id': 'abc-123', 'real': 'abc123'},
        {'id': '123-abc', 'real': 'B23abc'})
    @ddt.unpack
    def test_generate_share_name(self, id, real):
        ret = self.driver.generate_share_name({'id': id})
        self.assertEqual(real, ret)

    def test_get_network_allocations_number(self):
        ret = self.driver.get_network_allocations_number()
        self.assertEqual(0, ret)

    def test_create_share(self):
        mock_cs = self.mock_object(
            instorage.InStorageAssistant, 'create_share'
        )
        mock_gel = self.mock_object(
            instorage.InStorageAssistant,
            'get_export_locations',
            mock.Mock(return_value=['fake_export_location'])
        )

        ret = self.driver.create_share(self._ctxt, self.share_instance)

        self.assertEqual(['fake_export_location'], ret)
        mock_cs.assert_called_once_with('fakeinstanceid', 'P', 1, 'fake_proto')
        mock_gel.assert_called_once_with('fakeinstanceid', 'fake_proto')

    def test_delete_share(self):
        mock_ds = self.mock_object(
            instorage.InStorageAssistant, 'delete_share'
        )

        self.driver.delete_share(self._ctxt, self.share_instance)

        mock_ds.assert_called_once_with('fakeinstanceid', 'fake_proto')

    def test_extend_share(self):
        mock_es = self.mock_object(
            instorage.InStorageAssistant, 'extend_share'
        )

        self.driver.extend_share(self.share_instance, 3)

        mock_es.assert_called_once_with('fakeinstanceid', 3)

    def test_ensure_share(self):
        mock_gel = self.mock_object(
            instorage.InStorageAssistant,
            'get_export_locations',
            mock.Mock(return_value=['fake_export_location'])
        )

        ret = self.driver.ensure_share(self._ctxt, self.share_instance)

        self.assertEqual(['fake_export_location'], ret)
        mock_gel.assert_called_once_with('fakeinstanceid', 'fake_proto')

    def test_update_access(self):
        mock_ua = self.mock_object(
            instorage.InStorageAssistant, 'update_access'
        )

        self.driver.update_access(self._ctxt, self.share_instance, [], [], [])

        mock_ua.assert_called_once_with(
            'fakeinstanceid', 'fake_proto', [], [], []
        )


class FakeSSH(object):
    def __enter__(self):
        return self

    def __exit__(self, exec_type, exec_val, exec_tb):
        if exec_val:
            raise


class FakeSSHPool(object):
    def __init__(self, ssh):
        self.fakessh = ssh

    def item(self):
        return self.fakessh


class SSHRunnerTestCase(test.TestCase):
    def setUp(self):
        self.fakessh = FakeSSH()
        self.fakePool = FakeSSHPool(self.fakessh)
        super(SSHRunnerTestCase, self).setUp()

    def test___call___success(self):
        mock_csi = self.mock_object(manila_utils, 'check_ssh_injection')
        mock_sshpool = mock.Mock(return_value=self.fakePool)
        self.mock_object(manila_utils, 'SSHPool', mock_sshpool)
        mock_se = mock.Mock(return_value='fake_value')
        self.mock_object(cli_helper.SSHRunner, '_ssh_execute', mock_se)

        runner = cli_helper.SSHRunner(
            '127.0.0.1', '22', 'fakeuser', 'fakepassword'
        )
        ret = runner(['mcsinq', 'lsvdisk'])

        mock_csi.assert_called_once_with(['mcsinq', 'lsvdisk'])
        mock_sshpool.assert_called_once_with(
            '127.0.0.1', '22', 60, 'fakeuser',
            password='fakepassword',
            privatekey=None,
            min_size=1,
            max_size=10
        )
        mock_se.assert_called_once_with(
            self.fakePool,
            'mcsinq lsvdisk',
            True,
            1
        )
        self.assertEqual('fake_value', ret)

    def test___call___ssh_pool_failed(self):
        mock_csi = self.mock_object(manila_utils, 'check_ssh_injection')
        mock_sshpool = mock.Mock(side_effect=paramiko.SSHException())
        self.mock_object(manila_utils, 'SSHPool', mock_sshpool)

        runner = cli_helper.SSHRunner(
            '127.0.0.1', '22', 'fakeuser', 'fakepassword'
        )

        self.assertRaises(paramiko.SSHException, runner, ['mcsinq', 'lsvdisk'])
        mock_csi.assert_called_once_with(['mcsinq', 'lsvdisk'])

    def test___call___ssh_exec_failed(self):
        mock_csi = self.mock_object(manila_utils, 'check_ssh_injection')
        mock_sshpool = mock.Mock(return_value=self.fakePool)
        self.mock_object(manila_utils, 'SSHPool', mock_sshpool)
        exception = processutils.ProcessExecutionError()
        mock_se = mock.Mock(side_effect=exception)
        self.mock_object(cli_helper.SSHRunner, '_ssh_execute', mock_se)

        runner = cli_helper.SSHRunner(
            '127.0.0.1', '22', 'fakeuser', 'fakepassword'
        )

        self.assertRaises(
            processutils.ProcessExecutionError,
            runner,
            ['mcsinq', 'lsvdisk']
        )
        mock_csi.assert_called_once_with(['mcsinq', 'lsvdisk'])
        mock_sshpool.assert_called_once_with(
            '127.0.0.1', '22', 60, 'fakeuser',
            password='fakepassword',
            privatekey=None,
            min_size=1,
            max_size=10
        )

    def test__ssh_execute_success(self):
        mock_se = mock.Mock(return_value='fake_value')
        self.mock_object(processutils, 'ssh_execute', mock_se)

        runner = cli_helper.SSHRunner(
            '127.0.0.1', '22', 'fakeuser', 'fakepassword'
        )
        ret = runner._ssh_execute(self.fakePool, 'mcsinq lsvdisk')

        mock_se.assert_called_once_with(
            self.fakessh,
            'mcsinq lsvdisk',
            check_exit_code=True
        )
        self.assertEqual('fake_value', ret)

    def test__ssh_execute_success_run_again(self):
        mock_se = mock.Mock(side_effect=[Exception(), 'fake_value'])
        self.mock_object(processutils, 'ssh_execute', mock_se)
        mock_sleep = self.mock_object(greenthread, 'sleep')

        runner = cli_helper.SSHRunner(
            '127.0.0.1', '22', 'fakeuser', 'fakepassword'
        )
        ret = runner._ssh_execute(
            self.fakePool,
            'mcsinq lsvdisk',
            check_exit_code=True,
            attempts=2
        )

        call = mock.call(self.fakessh, 'mcsinq lsvdisk', check_exit_code=True)
        mock_se.assert_has_calls([call, call])
        mock_sleep.assert_called_once()
        self.assertEqual('fake_value', ret)

    def test__ssh_execute_failed_exec_failed(self):
        exception = Exception()
        exception.exit_code = '1'
        exception.stdout = 'fake_stdout'
        exception.stderr = 'fake_stderr'
        exception.cmd = 'fake_cmd_list'
        mock_se = mock.Mock(side_effect=exception)
        self.mock_object(processutils, 'ssh_execute', mock_se)
        mock_sleep = self.mock_object(greenthread, 'sleep')

        runner = cli_helper.SSHRunner(
            '127.0.0.1', '22', 'fakeuser', 'fakepassword'
        )

        self.assertRaises(
            processutils.ProcessExecutionError,
            runner._ssh_execute,
            self.fakePool,
            'mcsinq lsvdisk',
            check_exit_code=True,
            attempts=1
        )
        mock_se.assert_called_once_with(
            self.fakessh,
            'mcsinq lsvdisk',
            check_exit_code=True
        )
        mock_sleep.assert_called_once()

    def test__ssh_execute_failed_exec_failed_exception_error(self):
        mock_se = mock.Mock(side_effect=Exception())
        self.mock_object(processutils, 'ssh_execute', mock_se)
        mock_sleep = self.mock_object(greenthread, 'sleep')

        runner = cli_helper.SSHRunner(
            '127.0.0.1', '22', 'fakeuser', 'fakepassword'
        )

        self.assertRaises(
            processutils.ProcessExecutionError,
            runner._ssh_execute,
            self.fakePool,
            'mcsinq lsvdisk',
            check_exit_code=True,
            attempts=1
        )
        mock_se.assert_called_once_with(
            self.fakessh,
            'mcsinq lsvdisk',
            check_exit_code=True
        )
        mock_sleep.assert_called_once()


class CLIParserTestCase(test.TestCase):
    def test_cliparser_with_header(self):
        cmdlist = ['mcsinq', 'lsnasportip', '-delim', '!']
        response = [
            'head1!head2',
            'r1c1!r1c2',
            'r2c1!r2c2'
        ]
        response = '\n'.join(response)

        ret = cli_helper.CLIParser(
            response, cmdlist, delim='!', with_header=True
        )

        self.assertEqual(2, len(ret))
        self.assertEqual('r1c1', ret[0]['head1'])
        self.assertEqual('r1c2', ret[0]['head2'])
        self.assertEqual('r2c1', ret[1]['head1'])
        self.assertEqual('r2c2', ret[1]['head2'])

        value = [(v['head1'], v['head2']) for v in ret]
        self.assertEqual([('r1c1', 'r1c2'), ('r2c1', 'r2c2')], value)

    def test_cliparser_without_header(self):
        cmdlist = ['mcsinq', 'lsnasportip', '-delim', '!']
        response = [
            'head1!p1v1',
            'head2!p1v2',
            '',
            'head1!p2v1',
            'head2!p2v2'
        ]
        response = '\n'.join(response)

        ret = cli_helper.CLIParser(
            response, cmdlist, delim='!', with_header=False
        )

        self.assertEqual(2, len(ret))
        self.assertEqual('p1v1', ret[0]['head1'])
        self.assertEqual('p1v2', ret[0]['head2'])
        self.assertEqual('p2v1', ret[1]['head1'])
        self.assertEqual('p2v2', ret[1]['head2'])


@ddt.ddt
class InStorageSSHTestCase(test.TestCase):
    def setUp(self):
        self.sshMock = mock.Mock()
        self.ssh = cli_helper.InStorageSSH(self.sshMock)
        super(InStorageSSHTestCase, self).setUp()

    def tearDown(self):
        super(InStorageSSHTestCase, self).tearDown()

    @ddt.data(None, 'node1')
    def test_lsnode(self, node_id):
        if node_id:
            cmd = ['mcsinq', 'lsnode', '-delim', '!', node_id]
            response = [
                'id!1',
                'name!node1'
            ]
        else:
            cmd = ['mcsinq', 'lsnode', '-delim', '!']
            response = [
                'id!name',
                '1!node1',
                '2!node2'
            ]

        response = '\n'.join(response)
        self.sshMock.return_value = (response, '')

        ret = self.ssh.lsnode(node_id)

        if node_id:
            self.sshMock.assert_called_once_with(cmd)
            self.assertEqual('node1', ret[0]['name'])
        else:
            self.sshMock.assert_called_once_with(cmd)
            self.assertEqual('node1', ret[0]['name'])
            self.assertEqual('node2', ret[1]['name'])

    @ddt.data(None, 'Pool0')
    def test_lsnaspool(self, pool_id):
        response = [
            'pool_name!available_capacity',
            'Pool0!2GB'
        ]
        if pool_id is None:
            response.append('Pool1!3GB')

        response = '\n'.join(response)
        self.sshMock.return_value = (response, '')

        ret = self.ssh.lsnaspool(pool_id)

        if pool_id is None:
            cmd = ['mcsinq', 'lsnaspool', '-delim', '!']
            self.sshMock.assert_called_once_with(cmd)
            self.assertEqual('Pool0', ret[0]['pool_name'])
            self.assertEqual('2GB', ret[0]['available_capacity'])
            self.assertEqual('Pool1', ret[1]['pool_name'])
            self.assertEqual('3GB', ret[1]['available_capacity'])
        else:
            cmd = ['mcsinq', 'lsnaspool', '-delim', '!', pool_id]
            self.sshMock.assert_called_once_with(cmd)
            self.assertEqual('Pool0', ret[0]['pool_name'])
            self.assertEqual('2GB', ret[0]['available_capacity'])

    @ddt.data({'node_name': 'node1', 'fsname': 'fs1'},
              {'node_name': 'node1', 'fsname': None},
              {'node_name': None, 'fsname': 'fs1'},
              {'node_name': None, 'fsname': None})
    @ddt.unpack
    def test_lsfs(self, node_name, fsname):
        response = [
            'pool_name!fs_name!total_capacity!used_capacity',
            'pool0!fs0!10GB!1GB',
            'pool1!fs1!8GB!3GB'
        ]
        response = '\n'.join(response)
        self.sshMock.return_value = (response, '')

        if fsname and not node_name:
            self.assertRaises(exception.InvalidParameterValue,
                              self.ssh.lsfs,
                              node_name=node_name,
                              fsname=fsname)
        else:
            ret = self.ssh.lsfs(node_name, fsname)

            cmdlist = []
            if node_name and not fsname:
                cmdlist = ['mcsinq', 'lsfs', '-delim', '!', '-node', '"node1"']
            elif node_name and fsname:
                cmdlist = ['mcsinq', 'lsfs', '-delim', '!',
                           '-node', '"node1"', '-name', '"fs1"']
            else:
                cmdlist = ['mcsinq', 'lsfs', '-delim', '!', '-all']

            self.sshMock.assert_called_once_with(cmdlist)
            self.assertEqual('pool0', ret[0]['pool_name'])
            self.assertEqual('fs0', ret[0]['fs_name'])
            self.assertEqual('10GB', ret[0]['total_capacity'])
            self.assertEqual('1GB', ret[0]['used_capacity'])
            self.assertEqual('pool1', ret[1]['pool_name'])
            self.assertEqual('fs1', ret[1]['fs_name'])
            self.assertEqual('8GB', ret[1]['total_capacity'])
            self.assertEqual('3GB', ret[1]['used_capacity'])

    def test_addfs(self):
        self.sshMock.return_value = ('', '')

        self.ssh.addfs('fsname', 'fake_pool', 1, 'node1')

        cmdlist = ['mcsop', 'addfs', '-name', '"fsname"',
                   '-pool', '"fake_pool"', '-size', '1g', '-node', '"node1"']
        self.sshMock.assert_called_once_with(cmdlist)

    def test_rmfs(self):
        self.sshMock.return_value = ('', '')

        self.ssh.rmfs('fsname')

        cmdlist = ['mcsop', 'rmfs', '-name', '"fsname"']
        self.sshMock.assert_called_once_with(cmdlist)

    def test_expandfs(self):
        self.sshMock.return_value = ('', '')

        self.ssh.expandfs('fsname', 2)

        cmdlist = ['mcsop', 'expandfs', '-name', '"fsname"', '-size', '2g']
        self.sshMock.assert_called_once_with(cmdlist)

    def test_lsnasdir(self):
        response = [
            'parent_dir!name',
            '/fs/test_01!share_01'
        ]

        response = '\n'.join(response)
        self.sshMock.return_value = (response, '')

        ret = self.ssh.lsnasdir('/fs/test_01')

        cmdlist = ['mcsinq', 'lsnasdir', '-delim', '!', '"/fs/test_01"']
        self.sshMock.assert_called_once_with(cmdlist)
        self.assertEqual('/fs/test_01', ret[0]['parent_dir'])
        self.assertEqual('share_01', ret[0]['name'])

    def test_addnasdir(self):
        self.sshMock.return_value = ('', '')

        self.ssh.addnasdir('/fs/test_01/share_01')

        cmdlist = ['mcsop', 'addnasdir', '"/fs/test_01/share_01"']
        self.sshMock.assert_called_once_with(cmdlist)

    def test_chnasdir(self):
        self.sshMock.return_value = ('', '')

        self.ssh.chnasdir('/fs/test_01/share_01', '/fs/test_01/share_02')

        cmdlist = ['mcsop', 'chnasdir', '-oldpath', '"/fs/test_01/share_01"',
                   '-newpath', '"/fs/test_01/share_02"']
        self.sshMock.assert_called_once_with(cmdlist)

    def test_rmnasdir(self):
        self.sshMock.return_value = ('', '')

        self.ssh.rmnasdir('/fs/test_01/share_01')

        cmdlist = ['mcsop', 'rmnasdir', '"/fs/test_01/share_01"']
        self.sshMock.assert_called_once_with(cmdlist)

    def test_rmnfs(self):
        self.sshMock.return_value = ('', '')

        self.ssh.rmnfs('/fs/test_01/share_01')

        cmdlist = ['mcsop', 'rmnfs', '"/fs/test_01/share_01"']
        self.sshMock.assert_called_once_with(cmdlist)

    @ddt.data(None, '/fs/test_01')
    def test_lsnfslist(self, prefix):
        cmdlist = ['mcsinq', 'lsnfslist', '-delim', '!']
        if prefix:
            cmdlist.append('"/fs/test_01"')
        response = '\n'.join([
            'path',
            '/fs/test_01/share_01',
            '/fs/test_01/share_02'
        ])
        self.sshMock.return_value = (response, '')

        ret = self.ssh.lsnfslist(prefix)
        self.sshMock.assert_called_once_with(cmdlist)
        self.assertEqual('/fs/test_01/share_01', ret[0]['path'])
        self.assertEqual('/fs/test_01/share_02', ret[1]['path'])

    def test_lsnfsinfo(self):
        cmdlist = [
            'mcsinq', 'lsnfsinfo', '-delim', '!', '"/fs/test_01/share_01"'
        ]
        response = '\n'.join([
            'ip!mask!rights!root_squash!all_squash',
            '192.168.1.0!255.255.255.0!rw!root_squash!all_squash'
        ])
        self.sshMock.return_value = (response, '')

        ret = self.ssh.lsnfsinfo('/fs/test_01/share_01')

        self.sshMock.assert_called_once_with(cmdlist)
        self.assertEqual('192.168.1.0', ret[0]['ip'])
        self.assertEqual('255.255.255.0', ret[0]['mask'])
        self.assertEqual('rw', ret[0]['rights'])

    def test_addnfsclient(self):
        self.sshMock.return_value = ('', '')

        cmdlist = [
            'mcsop', 'addnfsclient', '-path', '"/fs/test_01/share_01"',
            '-client', '192.168.1.0/255.255.255.0:rw:ALL_SQUASH:ROOT_SQUASH'
        ]

        self.ssh.addnfsclient(
            '/fs/test_01/share_01',
            '192.168.1.0/255.255.255.0:rw:ALL_SQUASH:ROOT_SQUASH'
        )

        self.sshMock.assert_called_once_with(cmdlist)

    def test_chnfsclient(self):
        self.sshMock.return_value = ('', '')

        cmdlist = [
            'mcsop', 'chnfsclient', '-path', '"/fs/test_01/share_01"',
            '-client', '192.168.1.0/255.255.255.0:rw:ALL_SQUASH:ROOT_SQUASH'
        ]

        self.ssh.chnfsclient(
            '/fs/test_01/share_01',
            '192.168.1.0/255.255.255.0:rw:ALL_SQUASH:ROOT_SQUASH'
        )

        self.sshMock.assert_called_once_with(cmdlist)

    def test_rmnfsclient(self):
        self.sshMock.return_value = ('', '')

        cmdlist = [
            'mcsop', 'rmnfsclient', '-path', '"/fs/test_01/share_01"',
            '-client', '192.168.1.0/255.255.255.0'
        ]

        self.ssh.rmnfsclient(
            '/fs/test_01/share_01',
            '192.168.1.0/255.255.255.0:rw:ALL_SQUASH:ROOT_SQUASH'
        )

        self.sshMock.assert_called_once_with(cmdlist)

    @ddt.data(None, 'cifs')
    def test_lscifslist(self, filter):
        cmdlist = ['mcsinq', 'lscifslist', '-delim', '!']
        if filter:
            cmdlist.append('"%s"' % filter)
        response = '\n'.join([
            'name!path',
            'cifs!/fs/test_01/share_01'
        ])
        self.sshMock.return_value = (response, '')

        ret = self.ssh.lscifslist(filter)

        self.sshMock.assert_called_once_with(cmdlist)
        self.assertEqual('cifs', ret[0]['name'])
        self.assertEqual('/fs/test_01/share_01', ret[0]['path'])

    def test_lscifsinfo(self):
        cmdlist = ['mcsinq', 'lscifsinfo', '-delim', '!', '"cifs"']
        response = '\n'.join([
            'path!oplocks!type!name!rights',
            '/fs/test_01/share_01!on!LU!user1!rw'
        ])
        self.sshMock.return_value = (response, '')

        ret = self.ssh.lscifsinfo('cifs')

        self.sshMock.assert_called_once_with(cmdlist)
        self.assertEqual('/fs/test_01/share_01', ret[0]['path'])
        self.assertEqual('on', ret[0]['oplocks'])
        self.assertEqual('LU', ret[0]['type'])
        self.assertEqual('user1', ret[0]['name'])
        self.assertEqual('rw', ret[0]['rights'])

    def test_addcifs(self):
        self.sshMock.return_value = ('', '')

        cmdlist = [
            'mcsop', 'addcifs', '-name', 'cifs',
            '-path', '/fs/test_01/share_01', '-oplocks', 'off'
        ]

        self.ssh.addcifs('cifs', '/fs/test_01/share_01', 'off')
        self.sshMock.assert_called_once_with(cmdlist)

    def test_rmcifs(self):
        self.sshMock.return_value = ('', '')

        cmdlist = ['mcsop', 'rmcifs', 'cifs']

        self.ssh.rmcifs('cifs')
        self.sshMock.assert_called_once_with(cmdlist)

    def test_chcifs(self):
        self.sshMock.return_value = ('', '')

        cmdlist = ['mcsop', 'chcifs', '-name', 'cifs', '-oplocks', 'off']

        self.ssh.chcifs('cifs', 'off')
        self.sshMock.assert_called_once_with(cmdlist)

    def test_addcifsuser(self):
        self.sshMock.return_value = ('', '')

        cmdlist = [
            'mcsop', 'addcifsuser', '-name', 'cifs', '-rights', 'LU:user1:rw'
        ]

        self.ssh.addcifsuser('cifs', 'LU:user1:rw')
        self.sshMock.assert_called_once_with(cmdlist)

    def test_chcifsuser(self):
        self.sshMock.return_value = ('', '')

        cmdlist = [
            'mcsop', 'chcifsuser', '-name', 'cifs', '-rights', 'LU:user1:rw'
        ]

        self.ssh.chcifsuser('cifs', 'LU:user1:rw')
        self.sshMock.assert_called_once_with(cmdlist)

    def test_rmcifsuser(self):
        self.sshMock.return_value = ('', '')

        cmdlist = [
            'mcsop', 'rmcifsuser', '-name', 'cifs', '-rights', 'LU:user1'
        ]

        self.ssh.rmcifsuser('cifs', 'LU:user1:rw')
        self.sshMock.assert_called_once_with(cmdlist)

    def test_lsnasportip(self):
        cmdlist = ['mcsinq', 'lsnasportip', '-delim', '!']
        response = '\n'.join([
            'node_name!id!ip!mask!gw!link_state',
            'node1!1!192.168.10.1!255.255.255.0!192.168.10.254!active',
            'node2!1!192.168.10.2!255.255.255.0!192.168.10.254!inactive'
        ])

        self.sshMock.return_value = (response, '')

        ret = self.ssh.lsnasportip()

        self.sshMock.assert_called_once_with(cmdlist)
        self.assertEqual('node1', ret[0]['node_name'])
        self.assertEqual('1', ret[0]['id'])
        self.assertEqual('192.168.10.1', ret[0]['ip'])
        self.assertEqual('255.255.255.0', ret[0]['mask'])
        self.assertEqual('192.168.10.254', ret[0]['gw'])
        self.assertEqual('active', ret[0]['link_state'])
        self.assertEqual('node2', ret[1]['node_name'])
        self.assertEqual('1', ret[1]['id'])
        self.assertEqual('192.168.10.2', ret[1]['ip'])
        self.assertEqual('255.255.255.0', ret[1]['mask'])
        self.assertEqual('192.168.10.254', ret[1]['gw'])
        self.assertEqual('inactive', ret[1]['link_state'])


@ddt.ddt
class InStorageAssistantTestCase(test.TestCase):
    def setUp(self):
        self.sshMock = mock.Mock()
        self.assistant = instorage.InStorageAssistant(self.sshMock)
        super(InStorageAssistantTestCase, self).setUp()

    def tearDown(self):
        super(InStorageAssistantTestCase, self).tearDown()

    @ddt.data(
        {'size': '1000MB', 'gb_size': 1},
        {'size': '3GB', 'gb_size': 3},
        {'size': '4TB', 'gb_size': 4096},
        {'size': '5PB', 'gb_size': 5242880})
    @ddt.unpack
    def test_size_to_gb(self, size, gb_size):
        ret = self.assistant.size_to_gb(size)
        self.assertEqual(gb_size, ret)

    def test_get_available_pools(self):
        response_for_lsnaspool = ('\n'.join([
            'pool_name!available_capacity',
            'pool0!100GB',
            'pool1!150GB'
        ]), '')
        cmdlist = ['mcsinq', 'lsnaspool', '-delim', '!']
        self.sshMock.return_value = response_for_lsnaspool

        ret = self.assistant.get_available_pools()

        pools = ['pool0', 'pool1']
        self.assertEqual(pools, ret)
        self.sshMock.assert_called_once_with(cmdlist)

    def test_get_pools_attr(self):
        response_for_lsfs = ('\n'.join([
            'pool_name!fs_name!total_capacity!used_capacity',
            'pool0!fs0!10GB!1GB',
            'pool1!fs1!8GB!3GB'
        ]), '')
        call_for_lsfs = mock.call(['mcsinq', 'lsfs', '-delim', '!', '-all'])
        response_for_lsnaspool = ('\n'.join([
            'pool_name!available_capacity',
            'pool0!100GB',
            'pool1!150GB'
        ]), '')
        call_for_lsnaspool = mock.call(['mcsinq', 'lsnaspool', '-delim', '!'])
        self.sshMock.side_effect = [
            response_for_lsfs,
            response_for_lsnaspool
        ]

        ret = self.assistant.get_pools_attr(['pool0'])
        pools = {
            'pool0': {
                'pool_name': 'pool0',
                'total_capacity_gb': 110,
                'free_capacity_gb': 100,
                'allocated_capacity_gb': 10,
                'qos': False,
                'reserved_percentage': 0,
                'reserved_snapshot_percentage': 0,
                'dedupe': False,
                'compression': False,
                'thin_provisioning': False,
                'max_over_subscription_ratio': 0
            }
        }
        self.assertEqual(pools, ret)
        self.sshMock.assert_has_calls([call_for_lsfs, call_for_lsnaspool])

    def test_get_nodes_info(self):
        response_for_lsnasportip = ('\n'.join([
            'node_name!id!ip!mask!gw!link_state',
            'node1!1!192.168.10.1!255.255.255.0!192.168.10.254!active',
            'node2!1!192.168.10.2!255.255.255.0!192.168.10.254!inactive',
            'node1!2!!!!inactive',
            'node2!2!!!!inactive'
        ]), '')
        call_for_lsnasportip = mock.call([
            'mcsinq', 'lsnasportip', '-delim', '!'
        ])
        self.sshMock.side_effect = [response_for_lsnasportip]

        ret = self.assistant.get_nodes_info()
        nodes = {
            'node1': {
                '1': {
                    'node_name': 'node1',
                    'id': '1',
                    'ip': '192.168.10.1',
                    'mask': '255.255.255.0',
                    'gw': '192.168.10.254',
                    'link_state': 'active'
                }
            },
            'node2': {
                '1': {
                    'node_name': 'node2',
                    'id': '1',
                    'ip': '192.168.10.2',
                    'mask': '255.255.255.0',
                    'gw': '192.168.10.254',
                    'link_state': 'inactive'
                }
            }
        }
        self.assertEqual(nodes, ret)
        self.sshMock.assert_has_calls([call_for_lsnasportip])

    @ddt.data(
        {'name': '1' * 30, 'fsname': '1' * 30},
        {'name': '1' * 40, 'fsname': '1' * 32})
    @ddt.unpack
    def test_get_fsname_by_name(self, name, fsname):
        ret = self.assistant.get_fsname_by_name(name)

        self.assertEqual(fsname, ret)

    @ddt.data(
        {'name': '1' * 30, 'dirname': '1' * 30},
        {'name': '1' * 40, 'dirname': '1' * 32})
    @ddt.unpack
    def test_get_dirsname_by_name(self, name, dirname):
        ret = self.assistant.get_dirname_by_name(name)

        self.assertEqual(dirname, ret)

    @ddt.data(
        {'name': '1' * 30, 'dirpath': '/fs/' + '1' * 30 + '/' + '1' * 30},
        {'name': '1' * 40, 'dirpath': '/fs/' + '1' * 32 + '/' + '1' * 32})
    @ddt.unpack
    def test_get_dirpath_by_name(self, name, dirpath):
        ret = self.assistant.get_dirpath_by_name(name)

        self.assertEqual(dirpath, ret)

    @ddt.data('CIFS', 'NFS')
    def test_create_share(self, proto):
        response_for_lsnasportip = ('\n'.join([
            'node_name!id!ip!mask!gw!link_state',
            'node1!1!192.168.10.1!255.255.255.0!192.168.10.254!active'
        ]), '')
        call_for_lsnasportip = mock.call([
            'mcsinq', 'lsnasportip', '-delim', '!'
        ])
        response_for_addfs = ('', '')
        call_for_addfs = mock.call([
            'mcsop', 'addfs', '-name', '"fakename"', '-pool', '"fakepool"',
            '-size', '10g', '-node', '"node1"'
        ])
        response_for_addnasdir = ('', '')
        call_for_addnasdir = mock.call([
            'mcsop', 'addnasdir', '"/fs/fakename/fakename"'
        ])
        response_for_addcifs = ('', '')
        call_for_addcifs = mock.call([
            'mcsop', 'addcifs', '-name', 'fakename',
            '-path', '/fs/fakename/fakename', '-oplocks', 'off'
        ])

        side_effect = [
            response_for_lsnasportip,
            response_for_addfs,
            response_for_addnasdir
        ]
        calls = [call_for_lsnasportip, call_for_addfs, call_for_addnasdir]
        if proto == 'CIFS':
            side_effect.append(response_for_addcifs)
            calls.append(call_for_addcifs)
        self.sshMock.side_effect = side_effect

        self.assistant.create_share('fakename', 'fakepool', 10, proto)

        self.sshMock.assert_has_calls(calls)

    @ddt.data(True, False)
    def test_check_share_exist(self, exist):
        response_for_lsfs = ('\n'.join([
            'pool_name!fs_name!total_capacity!used_capacity',
            'pool0!fs0!10GB!1GB',
            'pool1!fs1!8GB!3GB'
        ]), '')
        call_for_lsfs = mock.call([
            'mcsinq', 'lsfs', '-delim', '!', '-all'
        ])
        self.sshMock.side_effect = [
            response_for_lsfs
        ]

        share_name = 'fs0' if exist else 'fs2'

        ret = self.assistant.check_share_exist(share_name)

        self.assertEqual(exist, ret)
        self.sshMock.assert_has_calls([call_for_lsfs])

    @ddt.data({'proto': 'CIFS', 'share_exist': False},
              {'proto': 'CIFS', 'share_exist': True},
              {'proto': 'NFS', 'share_exist': False},
              {'proto': 'NFS', 'share_exist': True})
    @ddt.unpack
    def test_delete_share(self, proto, share_exist):
        mock_cse = self.mock_object(
            instorage.InStorageAssistant,
            'check_share_exist',
            mock.Mock(return_value=share_exist)
        )
        response_for_rmcifs = ('', '')
        call_for_rmcifs = mock.call([
            'mcsop', 'rmcifs', 'fakename'
        ])
        response_for_rmnasdir = ('', '')
        call_for_rmnasdir = mock.call([
            'mcsop', 'rmnasdir', '"/fs/fakename/fakename"'
        ])
        response_for_rmfs = ('', '')
        call_for_rmfs = mock.call([
            'mcsop', 'rmfs', '-name', '"fakename"'
        ])

        side_effect = [response_for_rmnasdir, response_for_rmfs]
        calls = [call_for_rmnasdir, call_for_rmfs]
        if proto == 'CIFS':
            side_effect.insert(0, response_for_rmcifs)
            calls.insert(0, call_for_rmcifs)
        self.sshMock.side_effect = side_effect

        self.assistant.delete_share('fakename', proto)

        mock_cse.assert_called_once_with('fakename')
        if share_exist:
            self.sshMock.assert_has_calls(calls)
        else:
            self.sshMock.assert_not_called()

    def test_extend_share(self):
        response_for_lsfs = ('\n'.join([
            'pool_name!fs_name!total_capacity!used_capacity',
            'pool0!fs0!10GB!1GB',
            'pool1!fs1!8GB!3GB'
        ]), '')
        call_for_lsfs = mock.call([
            'mcsinq', 'lsfs', '-delim', '!', '-all'
        ])
        response_for_expandfs = ('', '')
        call_for_expandfs = mock.call([
            'mcsop', 'expandfs', '-name', '"fs0"', '-size', '2g'
        ])
        self.sshMock.side_effect = [response_for_lsfs, response_for_expandfs]

        self.assistant.extend_share('fs0', 12)

        self.sshMock.assert_has_calls([call_for_lsfs, call_for_expandfs])

    @ddt.data('CIFS', 'NFS')
    def test_get_export_locations(self, proto):
        response_for_lsnode = ('\n'.join([
            'id!name',
            '1!node1',
            '2!node2'
        ]), '')
        call_for_lsnode = mock.call([
            'mcsinq', 'lsnode', '-delim', '!'
        ])
        response_for_lsfs_node1 = ('\n'.join([
            'pool_name!fs_name!total_capacity!used_capacity',
            'pool0!fs0!10GB!1GB'
        ]), '')
        call_for_lsfs_node1 = mock.call([
            'mcsinq', 'lsfs', '-delim', '!', '-node', '"node1"'
        ])
        response_for_lsfs_node2 = ('\n'.join([
            'pool_name!fs_name!total_capacity!used_capacity',
            'pool1!fs1!10GB!1GB'
        ]), '')
        call_for_lsfs_node2 = mock.call([
            'mcsinq', 'lsfs', '-delim', '!', '-node', '"node2"'
        ])
        response_for_lsnasportip = ('\n'.join([
            'node_name!id!ip!mask!gw!link_state',
            'node1!1!192.168.10.1!255.255.255.0!192.168.10.254!active',
            'node1!2!192.168.10.2!255.255.255.0!192.168.10.254!active',
            'node1!3!!!!inactive',
            'node2!1!192.168.10.3!255.255.255.0!192.168.10.254!active',
            'node2!2!192.168.10.4!255.255.255.0!192.168.10.254!active',
            'node2!3!!!!inactive'
        ]), '')
        call_for_lsnasportip = mock.call([
            'mcsinq', 'lsnasportip', '-delim', '!'
        ])
        self.sshMock.side_effect = [
            response_for_lsnode,
            response_for_lsfs_node1,
            response_for_lsfs_node2,
            response_for_lsnasportip
        ]
        calls = [
            call_for_lsnode,
            call_for_lsfs_node1,
            call_for_lsfs_node2,
            call_for_lsnasportip
        ]

        ret = self.assistant.get_export_locations('fs1', proto)
        if proto == 'CIFS':
            locations = [
                {
                    'path': '\\\\192.168.10.3\\fs1',
                    'is_admin_only': False,
                    'metadata': {}
                },
                {
                    'path': '\\\\192.168.10.4\\fs1',
                    'is_admin_only': False,
                    'metadata': {}
                }
            ]
        else:
            locations = [
                {
                    'path': '192.168.10.3:/fs/fs1/fs1',
                    'is_admin_only': False,
                    'metadata': {}
                },
                {
                    'path': '192.168.10.4:/fs/fs1/fs1',
                    'is_admin_only': False,
                    'metadata': {}
                }
            ]
        self.assertEqual(locations, ret)
        self.sshMock.assert_has_calls(calls)

    def test_classify_nfs_client_spec_has_nfsinfo(self):
        response_for_lsnfslist = ('\n'.join([
            'path',
            '/fs/fs01/fs01'
        ]), '')
        call_for_lsnfslist = mock.call([
            'mcsinq', 'lsnfslist', '-delim', '!', '"/fs/fs01/fs01"'
        ])
        response_for_lsnfsinfo = ('\n'.join([
            'ip!mask!rights!all_squash!root_squash',
            '192.168.1.0!255.255.255.0!rw!all_squash!root_squash',
            '192.168.2.0!255.255.255.0!rw!all_squash!root_squash'
        ]), '')
        call_for_lsnfsinfo = mock.call([
            'mcsinq', 'lsnfsinfo', '-delim', '!', '"/fs/fs01/fs01"'
        ])
        self.sshMock.side_effect = [
            response_for_lsnfslist, response_for_lsnfsinfo
        ]
        calls = [call_for_lsnfslist, call_for_lsnfsinfo]

        client_spec = [
            '192.168.2.0/255.255.255.0:rw:all_squash:root_squash',
            '192.168.3.0/255.255.255.0:rw:all_squash:root_squash'
        ]
        add_spec, del_spec = self.assistant.classify_nfs_client_spec(
            client_spec, '/fs/fs01/fs01'
        )

        self.assertEqual(
            add_spec, ['192.168.3.0/255.255.255.0:rw:all_squash:root_squash']
        )
        self.assertEqual(
            del_spec, ['192.168.1.0/255.255.255.0:rw:all_squash:root_squash']
        )
        self.sshMock.assert_has_calls(calls)

    def test_classify_nfs_client_spec_has_no_nfsinfo(self):
        cmdlist = [
            'mcsinq', 'lsnfslist', '-delim', '!', '"/fs/fs01/fs01"'
        ]
        self.sshMock.return_value = ('', '')

        client_spec = [
            '192.168.2.0/255.255.255.0:rw:all_squash:root_squash',
        ]
        add_spec, del_spec = self.assistant.classify_nfs_client_spec(
            client_spec, '/fs/fs01/fs01'
        )

        self.assertEqual(client_spec, add_spec)
        self.assertEqual([], del_spec)
        self.sshMock.assert_called_once_with(cmdlist)

    def test_access_rule_to_client_spec(self):
        rule = {
            'access_type': 'ip',
            'access_to': '192.168.10.0/24',
            'access_level': 'rw'
        }

        ret = self.assistant.access_rule_to_client_spec(rule)

        spec = '192.168.10.0/255.255.255.0:rw:all_squash:root_squash'
        self.assertEqual(spec, ret)

    def test_access_rule_to_client_spec_type_failed(self):
        rule = {
            'access_type': 'user',
            'access_to': 'test01',
            'access_level': 'rw'
        }

        self.assertRaises(
            exception.ShareBackendException,
            self.assistant.access_rule_to_client_spec,
            rule
        )

    def test_access_rule_to_client_spec_ipversion_failed(self):
        rule = {
            'access_type': 'ip',
            'access_to': '2001:db8::/64',
            'access_level': 'rw'
        }

        self.assertRaises(
            exception.ShareBackendException,
            self.assistant.access_rule_to_client_spec,
            rule
        )

    @ddt.data(True, False)
    def test_update_nfs_access(self, check_del_add):
        response_for_rmnfsclient = ('', '')
        call_for_rmnfsclient = mock.call(
            ['mcsop', 'rmnfsclient', '-path', '"/fs/fs01/fs01"', '-client',
             '192.168.1.0/255.255.255.0']
        )
        response_for_addnfsclient = ('', '')
        call_for_addnfsclient = mock.call(
            ['mcsop', 'addnfsclient', '-path', '"/fs/fs01/fs01"', '-client',
             '192.168.3.0/255.255.255.0:rw:all_squash:root_squash']
        )
        access_rules = [
            {
                'access_type': 'ip',
                'access_to': '192.168.2.0/24',
                'access_level': 'rw'
            },
            {
                'access_type': 'ip',
                'access_to': '192.168.3.0/24',
                'access_level': 'rw'
            }
        ]
        add_rules = [
            {
                'access_type': 'ip',
                'access_to': '192.168.3.0/24',
                'access_level': 'rw'
            }
        ]
        del_rules = [
            {
                'access_type': 'ip',
                'access_to': '192.168.1.0/24',
                'access_level': 'rw'
            },
            {
                'access_type': 'ip',
                'access_to': '192.168.4.0/24',
                'access_level': 'rw'
            }
        ]

        cncs_mock = mock.Mock(return_value=(
            ['192.168.3.0/255.255.255.0:rw:all_squash:root_squash'],
            ['192.168.1.0/255.255.255.0:rw:all_squash:root_squash']
        ))
        self.mock_object(self.assistant, 'classify_nfs_client_spec', cncs_mock)
        self.sshMock.side_effect = [
            response_for_rmnfsclient, response_for_addnfsclient
        ]

        if check_del_add:
            self.assistant.update_nfs_access('fs01', [], add_rules, del_rules)
        else:
            self.assistant.update_nfs_access('fs01', access_rules, [], [])

        if check_del_add:
            cncs_mock.assert_called_once_with(
                [], '/fs/fs01/fs01'
            )
        else:
            cncs_mock.assert_called_once_with(
                [
                    '192.168.2.0/255.255.255.0:rw:all_squash:root_squash',
                    '192.168.3.0/255.255.255.0:rw:all_squash:root_squash'
                ],
                '/fs/fs01/fs01'
            )

        self.sshMock.assert_has_calls(
            [call_for_rmnfsclient, call_for_addnfsclient]
        )

    def test_classify_cifs_rights(self):
        cmdlist = ['mcsinq', 'lscifsinfo', '-delim', '!', '"fs01"']
        response_for_lscifsinfo = '\n'.join([
            'path!oplocks!type!name!rights',
            '/fs/fs01/fs01!on!LU!user1!rw',
            '/fs/fs01/fs01!on!LU!user2!rw'
        ])
        self.sshMock.return_value = (response_for_lscifsinfo, '')

        access_rights = [
            'LU:user2:rw',
            'LU:user3:rw'
        ]
        add_rights, del_rights = self.assistant.classify_cifs_rights(
            access_rights, 'fs01'
        )

        self.sshMock.assert_called_once_with(cmdlist)
        self.assertEqual(['LU:user3:rw'], add_rights)
        self.assertEqual(['LU:user1:rw'], del_rights)

    def test_access_rule_to_rights(self):
        rule = {
            'access_type': 'user',
            'access_to': 'test01',
            'access_level': 'rw'
        }

        ret = self.assistant.access_rule_to_rights(rule)
        self.assertEqual('LU:test01:rw', ret)

    def test_access_rule_to_rights_fail_type(self):
        rule = {
            'access_type': 'ip',
            'access_to': '192.168.1.0/24',
            'access_level': 'rw'
        }

        self.assertRaises(
            exception.ShareBackendException,
            self.assistant.access_rule_to_rights,
            rule
        )

    @ddt.data(True, False)
    def test_update_cifs_access(self, check_del_add):
        response_for_rmcifsuser = ('', None)
        call_for_rmcifsuser = mock.call(
            ['mcsop', 'rmcifsuser', '-name', 'fs01', '-rights', 'LU:user1']
        )
        response_for_addcifsuser = ('', None)
        call_for_addcifsuser = mock.call(
            ['mcsop', 'addcifsuser', '-name', 'fs01', '-rights', 'LU:user3:rw']
        )
        access_rules = [
            {
                'access_type': 'user',
                'access_to': 'user2',
                'access_level': 'rw'
            },
            {
                'access_type': 'user',
                'access_to': 'user3',
                'access_level': 'rw'
            }
        ]
        add_rules = [
            {
                'access_type': 'user',
                'access_to': 'user3',
                'access_level': 'rw'
            }
        ]
        del_rules = [
            {
                'access_type': 'user',
                'access_to': 'user1',
                'access_level': 'rw'
            }
        ]

        ccr_mock = mock.Mock(return_value=(['LU:user3:rw'], ['LU:user1:rw']))
        self.mock_object(self.assistant, 'classify_cifs_rights', ccr_mock)
        self.sshMock.side_effect = [
            response_for_rmcifsuser, response_for_addcifsuser
        ]

        if check_del_add:
            self.assistant.update_cifs_access('fs01', [], add_rules, del_rules)
        else:
            self.assistant.update_cifs_access('fs01', access_rules, [], [])

        if not check_del_add:
            ccr_mock.assert_called_once_with(
                ['LU:user2:rw', 'LU:user3:rw'], 'fs01'
            )

        self.sshMock.assert_has_calls(
            [call_for_rmcifsuser, call_for_addcifsuser]
        )

    def test_check_access_type(self):
        rules1 = {
            'access_type': 'ip',
            'access_to': '192.168.1.0/24',
            'access_level': 'rw'
        }
        rules2 = {
            'access_type': 'ip',
            'access_to': '192.168.2.0/24',
            'access_level': 'rw'
        }
        rules3 = {
            'access_type': 'user',
            'access_to': 'user1',
            'access_level': 'rw'
        }
        rules4 = {
            'access_type': 'user',
            'access_to': 'user2',
            'access_level': 'rw'
        }

        ret = self.assistant.check_access_type('ip', [rules1], [rules2])
        self.assertTrue(ret)
        ret = self.assistant.check_access_type('user', [rules3], [rules4])
        self.assertTrue(ret)
        ret = self.assistant.check_access_type('ip', [rules1], [rules3])
        self.assertFalse(ret)
        ret = self.assistant.check_access_type('user', [rules3], [rules1])
        self.assertFalse(ret)

    @ddt.data(
        {'proto': 'CIFS', 'ret': True},
        {'proto': 'CIFS', 'ret': False},
        {'proto': 'NFS', 'ret': True},
        {'proto': 'NFS', 'ret': False},
        {'proto': 'unknown', 'ret': True})
    @ddt.unpack
    def test_update_access(self, proto, ret):
        uca_mock = self.mock_object(
            self.assistant, 'update_cifs_access', mock.Mock()
        )
        una_mock = self.mock_object(
            self.assistant, 'update_nfs_access', mock.Mock()
        )
        cat_mock = self.mock_object(
            self.assistant, 'check_access_type', mock.Mock(return_value=ret)
        )

        if proto == 'unknown':
            self.assertRaises(
                exception.ShareBackendException,
                self.assistant.update_access,
                'fs01',
                proto,
                [],
                [],
                []
            )
            cat_mock.assert_not_called()
        elif ret is False:
            self.assertRaises(
                exception.InvalidShareAccess,
                self.assistant.update_access,
                'fs01',
                proto,
                [],
                [],
                []
            )
            cat_mock.assert_called_once()
        else:
            self.assistant.update_access(
                'fs01',
                proto,
                [],
                [],
                []
            )
            if proto == 'CIFS':
                uca_mock.assert_called_once_with('fs01', [], [], [])
                una_mock.assert_not_called()
            else:
                una_mock.assert_called_once_with('fs01', [], [], [])
                uca_mock.assert_not_called()
            cat_mock.assert_called_once()



# Copyright 2015 Hewlett Packard Enterprise Development LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""HPE 3PAR Driver for OpenStack Manila."""

import datetime
import hashlib
import inspect
import os
import re

from oslo_config import cfg
from oslo_config import types
from oslo_log import log
import six

from manila.common import config
from manila import exception
from manila.i18n import _
from manila.share import driver
from manila.share.drivers.hpe import hpe_3par_mediator
from manila.share import share_types
from manila.share import utils as share_utils
from manila import utils

LOG = log.getLogger(__name__)


class FPG(types.String, types.IPAddress):
    """FPG type.

    Used to represent multiple pools per backend values.
    Converts configuration value to an FPGs value.
    FPGs value format::

        FPG name, IP address 1, IP address 2, ..., IP address 4

    where FPG name is a string value,
    IP address is of type types.IPAddress

    Optionally doing range checking.
    If value is whitespace or empty string will raise error

    :param min_ip: Optional check that number of min IP address of VFS.
    :param max_ip: Optional check that number of max IP address of VFS.
    :param type_name: Type name to be used in the sample config file.

    """

    MAX_SUPPORTED_IP_PER_VFS = 4

    def __init__(self, min_ip=0, max_ip=MAX_SUPPORTED_IP_PER_VFS,
                 type_name='FPG'):
        types.String.__init__(self, type_name=type_name)
        types.IPAddress.__init__(self, type_name=type_name)

        if max_ip < min_ip:
            msg = _("Pool's max acceptable IP cannot be less than min.")
            raise exception.HPE3ParInvalid(err=msg)

        if min_ip < 0:
            msg = _("Pools must be configured with zero or more IPs.")
            raise exception.HPE3ParInvalid(err=msg)

        if max_ip > FPG.MAX_SUPPORTED_IP_PER_VFS:
            msg = (_("Pool's max acceptable IP cannot be greater than "
                     "supported value=%s.") % FPG.MAX_SUPPORTED_IP_PER_VFS)
            raise exception.HPE3ParInvalid(err=msg)

        self.min_ip = min_ip
        self.max_ip = max_ip

    def __call__(self, value):
        if value is None or value.strip(' ') == '':
            message = _("Invalid configuration. hpe3par_fpg must be set.")
            LOG.error(message)
            raise exception.HPE3ParInvalid(err=message)

        ips = []
        values = value.split(",")
        # Extract pool name
        pool_name = values.pop(0).strip()

        # values will now be ['ip1', ...]
        if len(values) < self.min_ip:
            msg = (_("Require at least %s IPs configured per "
                     "pool") % self.min_ip)
            raise exception.HPE3ParInvalid(err=msg)
        if len(values) > self.max_ip:
            msg = (_("Cannot configure IPs more than max supported "
                     "%s IPs per pool") % self.max_ip)
            raise exception.HPE3ParInvalid(err=msg)

        for ip_addr in values:
            ip_addr = types.String.__call__(self, ip_addr.strip())
            try:
                ips.append(types.IPAddress.__call__(self, ip_addr))
            except ValueError as verror:
                raise exception.HPE3ParInvalid(err=verror)
        fpg = {pool_name: ips}
        return fpg

    def __repr__(self):
        return 'FPG'

    def _formatter(self, value):
        return six.text_type(value)


HPE3PAR_OPTS = [
    cfg.StrOpt('hpe3par_api_url',
               default='',
               help="3PAR WSAPI Server Url like "
                    "https://<3par ip>:8080/api/v1"),
    cfg.StrOpt('hpe3par_username',
               default='',
               help="3PAR username with the 'edit' role"),
    cfg.StrOpt('hpe3par_password',
               default='',
               help="3PAR password for the user specified in hpe3par_username",
               secret=True),
    cfg.HostAddressOpt('hpe3par_san_ip',
                       help="IP address of SAN controller"),
    cfg.StrOpt('hpe3par_san_login',
               default='',
               help="Username for SAN controller"),
    cfg.StrOpt('hpe3par_san_password',
               default='',
               help="Password for SAN controller",
               secret=True),
    cfg.PortOpt('hpe3par_san_ssh_port',
                default=22,
                help='SSH port to use with SAN'),
    cfg.MultiOpt('hpe3par_fpg',
                 item_type=FPG(min_ip=0, max_ip=FPG.MAX_SUPPORTED_IP_PER_VFS),
                 help="The File Provisioning Group (FPG) to use"),
    cfg.BoolOpt('hpe3par_fstore_per_share',
                default=False,
                help="Use one filestore per share"),
    cfg.BoolOpt('hpe3par_require_cifs_ip',
                default=False,
                help="Require IP access rules for CIFS (in addition to user)"),
    cfg.BoolOpt('hpe3par_debug',
                default=False,
                help="Enable HTTP debugging to 3PAR"),
    cfg.StrOpt('hpe3par_cifs_admin_access_username',
               default='',
               help="File system admin user name for CIFS."),
    cfg.StrOpt('hpe3par_cifs_admin_access_password',
               default='',
               help="File system admin password for CIFS.",
               secret=True),
    cfg.StrOpt('hpe3par_cifs_admin_access_domain',
               default='LOCAL_CLUSTER',
               help="File system domain for the CIFS admin user."),
    cfg.StrOpt('hpe3par_share_mount_path',
               default='/mnt/',
               help="The path where shares will be mounted when deleting "
                    "nested file trees."),
]

CONF = cfg.CONF
CONF.register_opts(HPE3PAR_OPTS)


def to_list(var):
    """Convert var to list type if not"""
    if isinstance(var, six.string_types):
        return [var]
    else:
        return var


class HPE3ParShareDriver(driver.ShareDriver):
    """HPE 3PAR driver for Manila.

    Supports NFS and CIFS protocols on arrays with File Persona.

    Version history::

        1.0.0 - Begin Liberty development (post-Kilo)
        1.0.1 - Report thin/dedup/hp_flash_cache capabilities
        1.0.2 - Add share server/share network support
        2.0.0 - Rebranded HP to HPE
        2.0.1 - Add access_level (e.g. read-only support)
        2.0.2 - Add extend/shrink
        2.0.3 - Remove file tree on delete when using nested shares #1538800
        2.0.4 - Reduce the fsquota by share size
                when a share is deleted #1582931
        2.0.5 - Add update_access support
        2.0.6 - Multi pool support per backend
        2.0.7 - Fix get_vfs() to correctly validate conf IP addresses at
                boot up #1621016
        2.0.8 - Replace ConsistencyGroup with ShareGroup

    """

    VERSION = "2.0.8"

    def __init__(self, *args, **kwargs):
        super(HPE3ParShareDriver, self).__init__((True, False),
                                                 *args,
                                                 **kwargs)

        self.configuration = kwargs.get('configuration', None)
        self.configuration.append_config_values(HPE3PAR_OPTS)
        self.configuration.append_config_values(driver.ssh_opts)
        self.configuration.append_config_values(config.global_opts)
        self.fpgs = {}
        self._hpe3par = None  # mediator between driver and client

    def do_setup(self, context):
        """Any initialization the share driver does while starting."""

        LOG.info("Starting share driver %(driver_name)s (%(version)s)",
                 {'driver_name': self.__class__.__name__,
                  'version': self.VERSION})

        mediator = hpe_3par_mediator.HPE3ParMediator(
            hpe3par_username=self.configuration.hpe3par_username,
            hpe3par_password=self.configuration.hpe3par_password,
            hpe3par_api_url=self.configuration.hpe3par_api_url,
            hpe3par_debug=self.configuration.hpe3par_debug,
            hpe3par_san_ip=self.configuration.hpe3par_san_ip,
            hpe3par_san_login=self.configuration.hpe3par_san_login,
            hpe3par_san_password=self.configuration.hpe3par_san_password,
            hpe3par_san_ssh_port=self.configuration.hpe3par_san_ssh_port,
            hpe3par_fstore_per_share=(self.configuration
                                      .hpe3par_fstore_per_share),
            hpe3par_require_cifs_ip=self.configuration.hpe3par_require_cifs_ip,
            hpe3par_cifs_admin_access_username=(
                self.configuration.hpe3par_cifs_admin_access_username),
            hpe3par_cifs_admin_access_password=(
                self.configuration.hpe3par_cifs_admin_access_password),
            hpe3par_cifs_admin_access_domain=(
                self.configuration.hpe3par_cifs_admin_access_domain),
            hpe3par_share_mount_path=(
                self.configuration.hpe3par_share_mount_path),
            my_ip=self.configuration.my_ip,
            ssh_conn_timeout=self.configuration.ssh_conn_timeout,
        )

        mediator.do_setup()

        def _validate_pool_ips(addresses, conf_pool_ips):
            # Pool configured IP addresses should be subset of IP addresses
            # retured from vfs
            if not set(conf_pool_ips) <= set(addresses):
                msg = _("Incorrect configuration. "
                        "Configuration pool IP address did not match with "
                        "IP addresses at 3par array")
                raise exception.HPE3ParInvalid(err=msg)

        def _construct_fpg():
            # FPG must be configured and must exist.
            # self.configuration.safe_get('hpe3par_fpg') will have value in
            # following format:
            # [ {'pool_name':['ip_addr', 'ip_addr', ...]}, ... ]
            for fpg in self.configuration.safe_get('hpe3par_fpg'):
                pool_name = list(fpg)[0]
                conf_pool_ips = fpg[pool_name]

                # Validate the FPG and discover the VFS
                # This also validates the client, connection, firmware, WSAPI,
                # FPG...
                vfs_info = mediator.get_vfs(pool_name)
                if self.driver_handles_share_servers:
                    # Use discovered IP(s) from array
                    self.fpgs[pool_name] = {
                        vfs_info['vfsname']: vfs_info['vfsip']['address']}
                elif conf_pool_ips == []:
                    # not DHSS and IPs not configured in manila.conf.
                    if not vfs_info['vfsip']['address']:
                        msg = _("Unsupported configuration. "
                                "hpe3par_fpg must have IP address "
                                "or be discoverable at 3PAR")
                        LOG.error(msg)
                        raise exception.HPE3ParInvalid(err=msg)
                    else:
                        # Use discovered pool ips
                        self.fpgs[pool_name] = {
                            vfs_info['vfsname']: vfs_info['vfsip']['address']}
                else:
                    # not DHSS and IPs configured in manila.conf
                    _validate_pool_ips(vfs_info['vfsip']['address'],
                                       conf_pool_ips)
                    self.fpgs[pool_name] = {
                        vfs_info['vfsname']: conf_pool_ips}

        _construct_fpg()

        # Don't set _hpe3par until it is ready. Otherwise _update_stats fails.
        self._hpe3par = mediator

    def _get_pool_location_from_share_host(self, share_instance_host):
        # Return pool name, vfs, IPs for a pool from share instance host
        pool_name = share_utils.extract_host(share_instance_host, level='pool')
        if not pool_name:
            message = (_("Pool is not available in the share host %s.") %
                       share_instance_host)
            raise exception.InvalidHost(reason=message)

        if pool_name not in self.fpgs:
            message = (_("Pool location lookup failed. "
                         "Could not find pool %s") %
                       pool_name)
            raise exception.InvalidHost(reason=message)

        vfs = list(self.fpgs[pool_name])[0]
        ips = self.fpgs[pool_name][vfs]

        return (pool_name, vfs, ips)

    def _get_pool_location(self, share, share_server=None):
        # Return pool name, vfs, IPs for a pool from share host field
        # Use share_server if provided, instead of self.fpgs
        if share_server is not None:
            # When DHSS
            ips = share_server['backend_details'].get('ip')
            ips = to_list(ips)
            vfs = share_server['backend_details'].get('vfs')
            pool_name = share_server['backend_details'].get('fpg')
            return (pool_name, vfs, ips)
        else:
            # When DHSS = false
            return self._get_pool_location_from_share_host(share['host'])

    def check_for_setup_error(self):

        try:
            # Log the source SHA for support.  Only do this with DEBUG.
            if LOG.isEnabledFor(log.DEBUG):
                LOG.debug('HPE3ParShareDriver SHA1: %s',
                          self.sha1_hash(HPE3ParShareDriver))
                LOG.debug('HPE3ParMediator SHA1: %s',
                          self.sha1_hash(hpe_3par_mediator.HPE3ParMediator))
        except Exception as e:
            # Don't let any exceptions during the SHA1 logging interfere
            # with startup.  This is just debug info to identify the source
            # code.  If it doesn't work, just log a debug message.
            LOG.debug('Source code SHA1 not logged due to: %s',
                      six.text_type(e))

    @staticmethod
    def sha1_hash(clazz):
        """Get the SHA1 hash for the source of a class."""
        source_file = inspect.getsourcefile(clazz)
        file_size = os.path.getsize(source_file)

        sha1 = hashlib.sha1()
        sha1.update(("blob %u\0" % file_size).encode('utf-8'))

        with open(source_file, 'rb') as f:
            sha1.update(f.read())

        return sha1.hexdigest()

    def get_network_allocations_number(self):
        return 1

    def choose_share_server_compatible_with_share(self, context, share_servers,
                                                  share, snapshot=None,
                                                  share_group=None):
        """Method that allows driver to choose share server for provided share.

        If compatible share-server is not found, method should return None.

        :param context: Current context
        :param share_servers: list with share-server models
        :param share:  share model
        :param snapshot: snapshot model
        :param share_group: ShareGroup model with shares
        :returns: share-server or None
        """
        # If creating in a share group, raise exception
        if share_group:
            msg = _("HPE 3PAR driver does not support share group")
            raise exception.InvalidRequest(message=msg)

        pool_name = share_utils.extract_host(share['host'], level='pool')
        for share_server in share_servers:
            if share_server['backend_details'].get('fpg') == pool_name:
                return share_server
        return None

    @staticmethod
    def _validate_network_type(network_type):
        if network_type not in ('flat', 'vlan', None):
            reason = _('Invalid network type. %s is not supported by the '
                       '3PAR driver.')
            raise exception.NetworkBadConfigurationException(
                reason=reason % network_type)

    def _create_share_server(self, network_info, request_host=None):
        """Is called to create/setup share server"""
        # Return pool name, vfs, IPs for a pool
        pool_name, vfs, ips = self._get_pool_location_from_share_host(
            request_host)

        ip = network_info['network_allocations'][0]['ip_address']
        if ip not in ips:
            # Besides DHSS, admin could have setup IP to VFS directly on array
            if len(ips) > (FPG.MAX_SUPPORTED_IP_PER_VFS - 1):
                message = (_("Pool %s has exceeded 3PAR's "
                             "max supported VFS IP address") % pool_name)
                LOG.error(message)
                raise exception.Invalid(message)

            subnet = utils.cidr_to_netmask(network_info['cidr'])
            vlantag = network_info['segmentation_id']

            self._hpe3par.create_fsip(ip, subnet, vlantag, pool_name, vfs)
            # Update in global saved config, self.fpgs[pool_name]
            ips.append(ip)

        return {'share_server_name': network_info['server_id'],
                'share_server_id': network_info['server_id'],
                'ip': ip,
                'subnet': subnet,
                'vlantag': vlantag if vlantag else 0,
                'fpg': pool_name,
                'vfs': vfs}

    def _setup_server(self, network_info, metadata=None):

        LOG.debug("begin _setup_server with %s", network_info)

        self._validate_network_type(network_info['network_type'])
        if metadata is not None and metadata['request_host'] is not None:
            return self._create_share_server(network_info,
                                             metadata['request_host'])

    def _teardown_server(self, server_details, security_services=None):
        LOG.debug("begin _teardown_server with %s", server_details)
        fpg = server_details.get('fpg')
        vfs = server_details.get('vfs')
        ip = server_details.get('ip')
        self._hpe3par.remove_fsip(ip, fpg, vfs)
        if ip in self.fpgs[fpg][vfs]:
            self.fpgs[fpg][vfs].remove(ip)

    @staticmethod
    def build_share_comment(share):
        """Create an informational only comment to help admins and testers."""

        info = {
            'name': share['display_name'],
            'host': share['host'],
            'now': datetime.datetime.now().strftime('%H%M%S'),
        }

        acceptable = re.compile(r'[^a-zA-Z0-9_=:@# \-]+', re.UNICODE)
        comment = ("OpenStack Manila - host=%(host)s  orig_name=%(name)s "
                   "created=%(now)s" % info)

        return acceptable.sub('_', comment)[:254]  # clean and truncate

    def create_share(self, context, share, share_server=None):
        """Is called to create share."""

        fpg, vfs, ips = self._get_pool_location(share, share_server)

        protocol = share['share_proto']
        extra_specs = share_types.get_extra_specs_from_share(share)

        path = self._hpe3par.create_share(
            share['project_id'],
            share['id'],
            protocol,
            extra_specs,
            fpg, vfs,
            size=share['size'],
            comment=self.build_share_comment(share)
        )

        return self._hpe3par.build_export_locations(protocol, ips, path)

    def create_share_from_snapshot(self, context, share, snapshot,
                                   share_server=None, parent_share=None):
        """Is called to create share from snapshot."""

        fpg, vfs, ips = self._get_pool_location(share, share_server)

        protocol = share['share_proto']
        extra_specs = share_types.get_extra_specs_from_share(share)

        path = self._hpe3par.create_share_from_snapshot(
            share['id'],
            protocol,
            extra_specs,
            share['project_id'],
            snapshot['share_id'],
            snapshot['id'],
            fpg,
            vfs,
            ips,
            size=share['size'],
            comment=self.build_share_comment(share)
        )

        return self._hpe3par.build_export_locations(protocol, ips, path)

    def delete_share(self, context, share, share_server=None):
        """Deletes share and its fstore."""

        fpg, vfs, ips = self._get_pool_location(share, share_server)
        self._hpe3par.delete_share(share['project_id'],
                                   share['id'],
                                   share['size'],
                                   share['share_proto'],
                                   fpg,
                                   vfs,
                                   ips[0])

    def create_snapshot(self, context, snapshot, share_server=None):
        """Creates a snapshot of a share."""

        fpg, vfs, ips = self._get_pool_location(snapshot['share'],
                                                share_server)
        self._hpe3par.create_snapshot(snapshot['share']['project_id'],
                                      snapshot['share']['id'],
                                      snapshot['share']['share_proto'],
                                      snapshot['id'],
                                      fpg,
                                      vfs)

    def delete_snapshot(self, context, snapshot, share_server=None):
        """Deletes a snapshot of a share."""

        fpg, vfs, ips = self._get_pool_location(snapshot['share'],
                                                share_server)
        self._hpe3par.delete_snapshot(snapshot['share']['project_id'],
                                      snapshot['share']['id'],
                                      snapshot['share']['share_proto'],
                                      snapshot['id'],
                                      fpg,
                                      vfs)

    def ensure_share(self, context, share, share_server=None):
        pass

    def update_access(self, context, share, access_rules, add_rules,
                      delete_rules, share_server=None):
        """Update access to the share."""
        extra_specs = None
        if 'NFS' == share['share_proto']:  # Avoiding DB call otherwise
            extra_specs = share_types.get_extra_specs_from_share(share)

        fpg, vfs, ips = self._get_pool_location(share, share_server)
        self._hpe3par.update_access(share['project_id'],
                                    share['id'],
                                    share['share_proto'],
                                    extra_specs,
                                    access_rules,
                                    add_rules,
                                    delete_rules,
                                    fpg,
                                    vfs)

    def extend_share(self, share, new_size, share_server=None):
        """Extends size of existing share."""

        fpg, vfs, ips = self._get_pool_location(share, share_server)
        self._hpe3par.resize_share(share['project_id'],
                                   share['id'],
                                   share['share_proto'],
                                   new_size,
                                   share['size'],
                                   fpg,
                                   vfs)

    def shrink_share(self, share, new_size, share_server=None):
        """Shrinks size of existing share."""

        fpg, vfs, ips = self._get_pool_location(share, share_server)
        self._hpe3par.resize_share(share['project_id'],
                                   share['id'],
                                   share['share_proto'],
                                   new_size,
                                   share['size'],
                                   fpg,
                                   vfs)

    def _update_share_stats(self):
        """Retrieve stats info from share group."""

        backend_name = self.configuration.safe_get(
            'share_backend_name') or "HPE_3PAR"

        max_over_subscription_ratio = self.configuration.safe_get(
            'max_over_subscription_ratio')

        reserved_share_percentage = self.configuration.safe_get(
            'reserved_share_percentage')
        if reserved_share_percentage is None:
            reserved_share_percentage = 0

        reserved_share_from_snapshot_percentage = self.configuration.safe_get(
            'reserved_share_from_snapshot_percentage')
        if reserved_share_from_snapshot_percentage is None:
            reserved_share_from_snapshot_percentage = reserved_share_percentage

        stats = {
            'share_backend_name': backend_name,
            'driver_handles_share_servers': self.driver_handles_share_servers,
            'vendor_name': 'HPE',
            'driver_version': self.VERSION,
            'storage_protocol': 'NFS_CIFS',
            'total_capacity_gb': 0,
            'free_capacity_gb': 0,
            'provisioned_capacity_gb': 0,
            'reserved_percentage': reserved_share_percentage,
            'reserved_snapshot_percentage':
                reserved_share_from_snapshot_percentage,
            'max_over_subscription_ratio': max_over_subscription_ratio,
            'qos': False,
            'thin_provisioning': True,  # 3PAR default is thin
        }

        if not self._hpe3par:
            LOG.info(
                "Skipping capacity and capabilities update. Setup has not "
                "completed.")
        else:
            for fpg in self.fpgs:
                fpg_status = self._hpe3par.get_fpg_status(fpg)
                fpg_status['reserved_percentage'] = reserved_share_percentage
                fpg_status['reserved_snapshot_percentage'] = (
                    reserved_share_from_snapshot_percentage)
                LOG.debug("FPG status = %s.", fpg_status)
                stats.setdefault('pools', []).append(fpg_status)

        super(HPE3ParShareDriver, self)._update_share_stats(stats)



# Copyright 2010-2011 OpenStack, LLC
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Tests for database migrations.
"""

from unittest import mock

from alembic import script
from oslo_db.sqlalchemy import enginefacade
from oslo_db.sqlalchemy import test_fixtures
from oslo_db.sqlalchemy import test_migrations
from oslo_log import log
from oslotest import base as test_base
from sqlalchemy.sql import text

from manila.db.migrations.alembic import migration
from manila.tests.db.migrations.alembic import migrations_data_checks
from manila.tests import utils as test_utils

LOG = log.getLogger('manila.tests.test_migrations')


class ManilaMigrationsCheckers(test_migrations.WalkVersionsMixin,
                               migrations_data_checks.DbMigrationsData):
    """Test alembic migrations."""

    def setUp(self):
        super().setUp()
        self.engine = enginefacade.writer.get_engine()

    @property
    def snake_walk(self):
        return True

    @property
    def downgrade(self):
        return True

    @property
    def INIT_VERSION(self):
        pass

    @property
    def REPOSITORY(self):
        pass

    @property
    def migration_api(self):
        return migration

    @property
    def migrate_engine(self):
        return self.engine

    def _walk_versions(self, snake_walk=False, downgrade=True):
        # Determine latest version script from the repo, then
        # upgrade from 1 through to the latest, with no data
        # in the databases. This just checks that the schema itself
        # upgrades successfully.

        # Place the database under version control
        alembic_cfg = migration._alembic_config()
        script_directory = script.ScriptDirectory.from_config(alembic_cfg)

        self.assertIsNone(self.migration_api.version())

        versions = [ver for ver in script_directory.walk_revisions()]

        LOG.debug('latest version is %s', versions[0].revision)

        for version in reversed(versions):
            self._migrate_up(version.revision, with_data=True)
            if snake_walk:
                downgraded = self._migrate_down(
                    version, with_data=True)
                if downgraded:
                    self._migrate_up(version.revision)

        if downgrade:
            for version in versions:
                downgraded = self._migrate_down(version)
                if snake_walk and downgraded:
                    self._migrate_up(version.revision)
                    self._migrate_down(version)

    def _migrate_down(self, version, with_data=False):
        try:
            self.migration_api.downgrade(version.down_revision)
        except NotImplementedError:
            # NOTE(sirp): some migrations, namely release-level
            # migrations, don't support a downgrade.
            return False

        self.assertEqual(version.down_revision, self.migration_api.version())

        if with_data:
            post_downgrade = getattr(
                self, "_post_downgrade_%s" % version.revision, None)
            if post_downgrade:
                post_downgrade(self.engine)

        return True

    def _migrate_up(self, version, with_data=False):
        """migrate up to a new version of the db.

        We allow for data insertion and post checks at every
        migration version with special _pre_upgrade_### and
        _check_### functions in the main test.
        """
        # NOTE(sdague): try block is here because it's impossible to debug
        # where a failed data migration happens otherwise
        try:
            if with_data:
                data = None
                pre_upgrade = getattr(
                    self, "_pre_upgrade_%s" % version, None)
                if pre_upgrade:
                    data = pre_upgrade(self.engine)

            self.migration_api.upgrade(version)
            self.assertEqual(version, self.migration_api.version())
            if with_data:
                check = getattr(self, "_check_%s" % version, None)
                if check:
                    check(self.engine, data)
        except Exception as e:
            LOG.error("Failed to migrate to version %(version)s on engine "
                      "%(engine)s. Exception while running the migration: "
                      "%(exception)s", {'version': version,
                                        'engine': self.engine,
                                        'exception': e})
            raise

    # NOTE(vponomaryov): set 10 minutes timeout for case of running it on
    # very slow nodes/VMs. Note, that this test becomes slower with each
    # addition of new DB migration. On fast nodes it can take about 5-10 secs
    # having Mitaka set of migrations.
    # 'pymysql' works much slower on slow nodes than 'psycopg2'. And such
    # timeout mostly required for testing of 'mysql' backend.
    @test_utils.set_timeout(600)
    def test_walk_versions(self):
        """Walks all version scripts for each tested database.

        While walking, ensure that there are no errors in the version
        scripts for each engine.
        """
        with mock.patch('manila.db.sqlalchemy.api.get_engine',
                        return_value=self.engine):
            self._walk_versions(snake_walk=self.snake_walk,
                                downgrade=self.downgrade)

    def test_single_branch(self):
        alembic_cfg = migration._alembic_config()
        script_directory = script.ScriptDirectory.from_config(alembic_cfg)

        actual_result = script_directory.get_heads()

        self.assertEqual(1, len(actual_result),
                         "Db migrations should have only one branch.")


class TestManilaMigrationsMySQL(
    ManilaMigrationsCheckers,
    test_fixtures.OpportunisticDBTestMixin,
    test_base.BaseTestCase,
):
    """Run migration tests on MySQL backend."""
    FIXTURE = test_fixtures.MySQLOpportunisticFixture

    @test_utils.set_timeout(300)
    def test_mysql_innodb(self):
        """Test that table creation on mysql only builds InnoDB tables."""
        with mock.patch('manila.db.sqlalchemy.api.get_engine',
                        return_value=self.engine):
            self._walk_versions(snake_walk=False, downgrade=False)

        # sanity check
        sanity_check = """SELECT count(*)
                          FROM information_schema.tables
                          WHERE table_schema = :database;"""
        total = self.engine.execute(
            text(sanity_check),
            database=self.engine.url.database)

        self.assertGreater(total.scalar(), 0, "No tables found. Wrong schema?")

        noninnodb_query = """
            SELECT count(*)
            FROM information_schema.TABLES
            WHERE table_schema = :database
                AND engine != 'InnoDB'
                AND table_name != 'alembic_version';"""

        count = self.engine.execute(
            text(noninnodb_query),
            database=self.engine.url.database
        ).scalar()
        self.assertEqual(0, count, "%d non InnoDB tables created" % count)


class TestManilaMigrationsPostgreSQL(
    ManilaMigrationsCheckers,
    test_fixtures.OpportunisticDBTestMixin,
    test_base.BaseTestCase,
):
    """Run migration tests on PostgreSQL backend."""
    FIXTURE = test_fixtures.PostgresqlOpportunisticFixture



# Copyright (c) 2015 Cloudbase Solutions SRL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
from unittest import mock

import ddt

from manila.common import constants
from manila import exception
from manila.share import configuration
from manila.share.drivers.windows import windows_smb_helper
from manila.share.drivers.windows import windows_utils
from manila import test

from oslo_config import cfg

CONF = cfg.CONF
CONF.import_opt('share_mount_path',
                'manila.share.drivers.generic')


@ddt.ddt
class WindowsSMBHelperTestCase(test.TestCase):
    _FAKE_SERVER = {'public_address': mock.sentinel.public_address}
    _FAKE_SHARE_NAME = "fake_share_name"
    _FAKE_SHARE = "\\\\%s\\%s" % (_FAKE_SERVER['public_address'],
                                  _FAKE_SHARE_NAME)
    _FAKE_SHARE_LOCATION = os.path.join(
        configuration.Configuration(None).share_mount_path,
        _FAKE_SHARE_NAME)
    _FAKE_ACCOUNT_NAME = 'FakeDomain\\FakeUser'
    _FAKE_RW_ACC_RULE = {
        'access_to': _FAKE_ACCOUNT_NAME,
        'access_level': constants.ACCESS_LEVEL_RW,
        'access_type': 'user',
    }

    def setUp(self):
        self._remote_exec = mock.Mock()
        fake_conf = configuration.Configuration(None)

        self._win_smb_helper = windows_smb_helper.WindowsSMBHelper(
            self._remote_exec, fake_conf)

        super(WindowsSMBHelperTestCase, self).setUp()

    def test_init_helper(self):
        self._win_smb_helper.init_helper(mock.sentinel.server)
        self._remote_exec.assert_called_once_with(mock.sentinel.server,
                                                  "Get-SmbShare")

    @ddt.data(True, False)
    @mock.patch.object(windows_smb_helper.WindowsSMBHelper, '_share_exists')
    def test_create_exports(self, share_exists, mock_share_exists):
        mock_share_exists.return_value = share_exists

        result = self._win_smb_helper.create_exports(self._FAKE_SERVER,
                                                     self._FAKE_SHARE_NAME)

        if not share_exists:
            cmd = ['New-SmbShare', '-Name', self._FAKE_SHARE_NAME, '-Path',
                   self._win_smb_helper._windows_utils.normalize_path(
                       self._FAKE_SHARE_LOCATION),
                   '-ReadAccess', "*%s" % self._win_smb_helper._NULL_SID]
            self._remote_exec.assert_called_once_with(self._FAKE_SERVER, cmd)
        else:
            self.assertFalse(self._remote_exec.called)

        expected_exports = [
            {
                'is_admin_only': False,
                'metadata': {'export_location_metadata_example': 'example'},
                'path': self._FAKE_SHARE
            },
        ]

        self.assertEqual(expected_exports, result)

    @mock.patch.object(windows_smb_helper.WindowsSMBHelper, '_share_exists')
    def test_remove_exports(self, mock_share_exists):
        mock_share_exists.return_value = True

        self._win_smb_helper.remove_exports(mock.sentinel.server,
                                            mock.sentinel.share_name)

        cmd = ['Remove-SmbShare', '-Name', mock.sentinel.share_name, "-Force"]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    @mock.patch.object(windows_utils.WindowsUtils,
                       'get_volume_path_by_mount_path')
    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_get_share_path_by_name')
    def test_get_volume_path_by_share_name(self, mock_get_share_path,
                                           mock_get_vol_path):
        mock_get_share_path.return_value = self._FAKE_SHARE_LOCATION

        volume_path = self._win_smb_helper._get_volume_path_by_share_name(
            mock.sentinel.server, self._FAKE_SHARE_NAME)

        mock_get_share_path.assert_called_once_with(mock.sentinel.server,
                                                    self._FAKE_SHARE_NAME)
        mock_get_vol_path.assert_called_once_with(mock.sentinel.server,
                                                  self._FAKE_SHARE_LOCATION)

        self.assertEqual(mock_get_vol_path.return_value, volume_path)

    @ddt.data({'raw_out': '', 'expected': []},
              {'raw_out': '{"key": "val"}',
               'expected': [{"key": "val"}]},
              {'raw_out': '[{"key": "val"}, {"key2": "val2"}]',
               'expected': [{"key": "val"}, {"key2": "val2"}]})
    @ddt.unpack
    def test_get_acls_helper(self, raw_out, expected):
        self._remote_exec.return_value = (raw_out, mock.sentinel.err)

        rules = self._win_smb_helper._get_acls(mock.sentinel.server,
                                               self._FAKE_SHARE_NAME)

        self.assertEqual(expected, rules)
        expected_cmd = (
            'Get-SmbShareAccess -Name %s | '
            'Select-Object @("Name", "AccountName", '
            '"AccessControlType", "AccessRight") | '
            'ConvertTo-JSON -Compress') % self._FAKE_SHARE_NAME
        self._remote_exec.assert_called_once_with(mock.sentinel.server,
                                                  expected_cmd)

    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_get_acls')
    def test_get_access_rules(self, mock_get_acls):
        helper = self._win_smb_helper
        valid_acl = {
            'AccountName': self._FAKE_ACCOUNT_NAME,
            'AccessRight': helper._WIN_ACCESS_RIGHT_FULL,
            'AccessControlType': helper._WIN_ACL_ALLOW,
        }

        valid_acls = [valid_acl,
                      dict(valid_acl,
                           AccessRight=helper._WIN_ACCESS_RIGHT_CHANGE),
                      dict(valid_acl,
                           AccessRight=helper._WIN_ACCESS_RIGHT_READ)]
        # Those are rules that were not added by us and are expected to
        # be ignored. When encountering such a rule, a warning message
        # will be logged.
        ignored_acls = [
            dict(valid_acl, AccessRight=helper._WIN_ACCESS_RIGHT_CUSTOM),
            dict(valid_acl, AccessControlType=helper._WIN_ACL_DENY)]

        mock_get_acls.return_value = valid_acls + ignored_acls
        # There won't be multiple access rules for the same account,
        # but we'll ignore this fact for the sake of this test.
        expected_rules = [self._FAKE_RW_ACC_RULE, self._FAKE_RW_ACC_RULE,
                          dict(self._FAKE_RW_ACC_RULE,
                               access_level=constants.ACCESS_LEVEL_RO)]

        rules = helper.get_access_rules(mock.sentinel.server,
                                        mock.sentinel.share_name)
        self.assertEqual(expected_rules, rules)

        mock_get_acls.assert_called_once_with(mock.sentinel.server,
                                              mock.sentinel.share_name)

    @mock.patch.object(windows_smb_helper.WindowsSMBHelper, '_refresh_acl')
    def test_grant_share_access(self, mock_refresh_acl):
        self._win_smb_helper._grant_share_access(mock.sentinel.server,
                                                 mock.sentinel.share_name,
                                                 constants.ACCESS_LEVEL_RW,
                                                 mock.sentinel.username)

        cmd = ["Grant-SmbShareAccess", "-Name", mock.sentinel.share_name,
               "-AccessRight", "Change",
               "-AccountName", "'%s'" % mock.sentinel.username, "-Force"]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        mock_refresh_acl.assert_called_once_with(mock.sentinel.server,
                                                 mock.sentinel.share_name)

    def test_refresh_acl(self):
        self._win_smb_helper._refresh_acl(mock.sentinel.server,
                                          mock.sentinel.share_name)

        cmd = ['Set-SmbPathAcl', '-ShareName', mock.sentinel.share_name]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    @mock.patch.object(windows_smb_helper.WindowsSMBHelper, '_refresh_acl')
    def test_revoke_share_access(self, mock_refresh_acl):
        self._win_smb_helper._revoke_share_access(mock.sentinel.server,
                                                  mock.sentinel.share_name,
                                                  mock.sentinel.username)

        cmd = ["Revoke-SmbShareAccess", "-Name", mock.sentinel.share_name,
               "-AccountName", '"%s"' % mock.sentinel.username, "-Force"]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        mock_refresh_acl.assert_called_once_with(mock.sentinel.server,
                                                 mock.sentinel.share_name)

    def test_update_access_invalid_type(self):
        invalid_access_rule = dict(self._FAKE_RW_ACC_RULE,
                                   access_type='ip')
        self.assertRaises(
            exception.InvalidShareAccess,
            self._win_smb_helper.update_access,
            mock.sentinel.server, mock.sentinel.share_name,
            [invalid_access_rule], [], [])

    def test_update_access_invalid_level(self):
        invalid_access_rule = dict(self._FAKE_RW_ACC_RULE,
                                   access_level='fake_level')
        self.assertRaises(
            exception.InvalidShareAccessLevel,
            self._win_smb_helper.update_access,
            mock.sentinel.server, mock.sentinel.share_name,
            [], [invalid_access_rule], [])

    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_revoke_share_access')
    def test_update_access_deleting_invalid_rule(self, mock_revoke):
        # We want to make sure that we allow deleting invalid rules.
        invalid_access_rule = dict(self._FAKE_RW_ACC_RULE,
                                   access_level='fake_level')
        delete_rules = [invalid_access_rule, self._FAKE_RW_ACC_RULE]

        self._win_smb_helper.update_access(
            mock.sentinel.server, mock.sentinel.share_name,
            [], [], delete_rules)

        mock_revoke.assert_called_once_with(
            mock.sentinel.server, mock.sentinel.share_name,
            self._FAKE_RW_ACC_RULE['access_to'])

    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       'validate_access_rules')
    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       'get_access_rules')
    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_grant_share_access')
    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_revoke_share_access')
    def test_update_access(self, mock_revoke, mock_grant,
                           mock_get_access_rules, mock_validate):
        added_rules = [mock.MagicMock(), mock.MagicMock()]
        deleted_rules = [mock.MagicMock(), mock.MagicMock()]

        self._win_smb_helper.update_access(
            mock.sentinel.server, mock.sentinel.share_name,
            [], added_rules, deleted_rules)

        mock_revoke.assert_has_calls(
            [mock.call(mock.sentinel.server, mock.sentinel.share_name,
                       deleted_rule['access_to'])
             for deleted_rule in deleted_rules])

        mock_grant.assert_has_calls(
            [mock.call(mock.sentinel.server, mock.sentinel.share_name,
                       added_rule['access_level'], added_rule['access_to'])
             for added_rule in added_rules])

    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_get_rule_updates')
    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       'validate_access_rules')
    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       'get_access_rules')
    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_grant_share_access')
    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_revoke_share_access')
    def test_update_access_maintenance(
            self, mock_revoke, mock_grant,
            mock_get_access_rules, mock_validate,
            mock_get_rule_updates):
        all_rules = mock.MagicMock()
        added_rules = [mock.MagicMock(), mock.MagicMock()]
        deleted_rules = [mock.MagicMock(), mock.MagicMock()]

        mock_get_rule_updates.return_value = [
            added_rules, deleted_rules]

        self._win_smb_helper.update_access(
            mock.sentinel.server, mock.sentinel.share_name,
            all_rules, [], [])

        mock_get_access_rules.assert_called_once_with(
            mock.sentinel.server, mock.sentinel.share_name)
        mock_get_rule_updates.assert_called_once_with(
            existing_rules=mock_get_access_rules.return_value,
            requested_rules=all_rules)
        mock_revoke.assert_has_calls(
            [mock.call(mock.sentinel.server, mock.sentinel.share_name,
                       deleted_rule['access_to'])
             for deleted_rule in deleted_rules])

        mock_grant.assert_has_calls(
            [mock.call(mock.sentinel.server, mock.sentinel.share_name,
                       added_rule['access_level'], added_rule['access_to'])
             for added_rule in added_rules])

    def test_get_rule_updates(self):
        req_rule_0 = self._FAKE_RW_ACC_RULE
        req_rule_1 = dict(self._FAKE_RW_ACC_RULE,
                          access_to='fake_acc')

        curr_rule_0 = dict(self._FAKE_RW_ACC_RULE,
                           access_to=self._FAKE_RW_ACC_RULE[
                               'access_to'].upper())
        curr_rule_1 = dict(self._FAKE_RW_ACC_RULE,
                           access_to='fake_acc2')
        curr_rule_2 = dict(req_rule_1,
                           access_level=constants.ACCESS_LEVEL_RO)

        expected_added_rules = [req_rule_1]
        expected_deleted_rules = [curr_rule_1, curr_rule_2]

        existing_rules = [curr_rule_0, curr_rule_1, curr_rule_2]
        requested_rules = [req_rule_0, req_rule_1]

        (added_rules,
         deleted_rules) = self._win_smb_helper._get_rule_updates(
            existing_rules, requested_rules)

        self.assertEqual(expected_added_rules, added_rules)
        self.assertEqual(expected_deleted_rules, deleted_rules)

    def test_get_share_name(self):
        result = self._win_smb_helper._get_share_name(self._FAKE_SHARE)
        self.assertEqual(self._FAKE_SHARE_NAME, result)

    def test_get_share_path_by_name(self):
        self._remote_exec.return_value = (self._FAKE_SHARE_LOCATION,
                                          mock.sentinel.std_err)

        result = self._win_smb_helper._get_share_path_by_name(
            mock.sentinel.server,
            mock.sentinel.share_name)

        cmd = ('Get-SmbShare -Name %s | '
               'Select-Object -ExpandProperty Path' % mock.sentinel.share_name)
        self._remote_exec.assert_called_once_with(mock.sentinel.server,
                                                  cmd,
                                                  check_exit_code=True)
        self.assertEqual(self._FAKE_SHARE_LOCATION, result)

    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_get_share_path_by_name')
    def test_get_share_path_by_export_location(self,
                                               mock_get_share_path_by_name):
        mock_get_share_path_by_name.return_value = mock.sentinel.share_path

        result = self._win_smb_helper.get_share_path_by_export_location(
            mock.sentinel.server, self._FAKE_SHARE)

        mock_get_share_path_by_name.assert_called_once_with(
            mock.sentinel.server, self._FAKE_SHARE_NAME)
        self.assertEqual(mock.sentinel.share_path, result)

    @mock.patch.object(windows_smb_helper.WindowsSMBHelper,
                       '_get_share_path_by_name')
    def test_share_exists(self, mock_get_share_path_by_name):
        result = self._win_smb_helper._share_exists(mock.sentinel.server,
                                                    mock.sentinel.share_name)

        mock_get_share_path_by_name.assert_called_once_with(
            mock.sentinel.server,
            mock.sentinel.share_name,
            ignore_missing=True)
        self.assertTrue(result)



# Copyright 2015 Alex Meade
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manila.api import common


class ShareGroupSnapshotViewBuilder(common.ViewBuilder):
    """Model a share group snapshot API response as a python dictionary."""

    _collection_name = "share_group_snapshot"

    def summary_list(self, request, group_snaps):
        """Show a list of share_group_snapshots without many details."""
        return self._list_view(self.summary, request, group_snaps)

    def detail_list(self, request, group_snaps):
        """Detailed view of a list of share_group_snapshots."""
        return self._list_view(self.detail, request, group_snaps)

    def member_list(self, request, members):
        members_list = []
        for member in members:
            member_dict = {
                'id': member.get('id'),
                'created_at': member.get('created_at'),
                'size': member.get('size'),
                'share_protocol': member.get('share_proto'),
                'project_id': member.get('project_id'),
                'share_group_snapshot_id': member.get(
                    'share_group_snapshot_id'),
                'share_id': member.get('share_instance', {}).get('share_id'),
                # TODO(vponomaryov): add 'provider_location' key in Pike.
            }
            members_list.append(member_dict)

        members_links = self._get_collection_links(
            request, members, "share_group_snapshot_id")
        members_dict = {"share_group_snapshot_members": members_list}

        if members_links:
            members_dict["share_group_snapshot_members_links"] = members_links

        return members_dict

    def summary(self, request, share_group_snap):
        """Generic, non-detailed view of a share group snapshot."""
        return {
            'share_group_snapshot': {
                'id': share_group_snap.get('id'),
                'name': share_group_snap.get('name'),
                'links': self._get_links(request, share_group_snap['id']),
            }
        }

    def detail(self, request, share_group_snap):
        """Detailed view of a single share group snapshot."""

        members = self._format_member_list(
            share_group_snap.get('share_group_snapshot_members', []))

        share_group_snap_dict = {
            'id': share_group_snap.get('id'),
            'name': share_group_snap.get('name'),
            'created_at': share_group_snap.get('created_at'),
            'status': share_group_snap.get('status'),
            'description': share_group_snap.get('description'),
            'project_id': share_group_snap.get('project_id'),
            'share_group_id': share_group_snap.get('share_group_id'),
            'members': members,
            'links': self._get_links(request, share_group_snap['id']),
        }
        return {'share_group_snapshot': share_group_snap_dict}

    def _format_member_list(self, members):
        members_list = []
        for member in members:
            member_dict = {
                'id': member.get('id'),
                'size': member.get('size'),
                'share_id': member.get('share_instance', {}).get('share_id'),
            }
            members_list.append(member_dict)

        return members_list

    def _list_view(self, func, request, snaps):
        """Provide a view for a list of share group snapshots."""
        snap_list = [func(request, snap)["share_group_snapshot"]
                     for snap in snaps]
        snaps_links = self._get_collection_links(request,
                                                 snaps,
                                                 self._collection_name)
        snaps_dict = {"share_group_snapshots": snap_list}

        if snaps_links:
            snaps_dict["share_group_snapshot_links"] = snaps_links

        return snaps_dict



# Copyright (c) 2014 OpenStack Foundation.
# Copyright (c) 2015 Tom Barron.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Built-in share type properties."""

import re

from oslo_config import cfg
from oslo_db import exception as db_exception
from oslo_log import log
from oslo_utils import strutils
from oslo_utils import uuidutils
import six

from manila.api import common
from manila.common import constants
from manila import context
from manila import db
from manila import exception
from manila.i18n import _

CONF = cfg.CONF
LOG = log.getLogger(__name__)

MIN_SIZE_KEY = "provisioning:min_share_size"
MAX_SIZE_KEY = "provisioning:max_share_size"


def create(context, name, extra_specs=None, is_public=True,
           projects=None, description=None):
    """Creates share types."""
    extra_specs = extra_specs or {}
    projects = projects or []

    try:
        get_valid_required_extra_specs(extra_specs)
        get_valid_optional_extra_specs(extra_specs)
    except exception.InvalidExtraSpec as e:
        raise exception.InvalidShareType(reason=six.text_type(e))

    extra_specs = sanitize_extra_specs(extra_specs)

    try:
        type_ref = db.share_type_create(context,
                                        dict(name=name,
                                             description=description,
                                             extra_specs=extra_specs,
                                             is_public=is_public),
                                        projects=projects)
    except db_exception.DBError:
        LOG.exception('DB error.')
        raise exception.ShareTypeCreateFailed(name=name,
                                              extra_specs=extra_specs)
    return type_ref


def sanitize_extra_specs(extra_specs):
    """Post process extra specs here if necessary"""
    az_spec = constants.ExtraSpecs.AVAILABILITY_ZONES
    if az_spec in extra_specs:
        extra_specs[az_spec] = sanitize_csv(extra_specs[az_spec])
    return extra_specs


def update(context, id, name, description, is_public=None):
    """Update share type by id."""
    values = {}
    if name:
        values.update({'name': name})
    if description == "":
        values.update({'description': None})
    elif description:
        values.update({'description': description})
    if is_public is not None:
        values.update({'is_public': is_public})
    try:
        db.share_type_update(context, id, values)
    except db_exception.DBError:
        LOG.exception('DB error.')
        raise exception.ShareTypeUpdateFailed(id=id)


def destroy(context, id):
    """Marks share types as deleted."""
    if id is None:
        msg = _("id cannot be None")
        raise exception.InvalidShareType(reason=msg)
    else:
        db.share_type_destroy(context, id)


def get_all_types(context, inactive=0, search_opts=None):
    """Get all non-deleted share_types.

    """
    search_opts = search_opts or {}
    filters = {}

    if 'is_public' in search_opts:
        filters['is_public'] = search_opts.pop('is_public')

    share_types = db.share_type_get_all(context, inactive, filters=filters)

    for type_name, type_args in share_types.items():
        required_extra_specs = {}
        try:
            required_extra_specs = get_valid_required_extra_specs(
                type_args['extra_specs'])
        except exception.InvalidExtraSpec:
            LOG.exception('Share type %(share_type)s has invalid required'
                          ' extra specs.', {'share_type': type_name})

        type_args['required_extra_specs'] = required_extra_specs

    search_vars = {}
    availability_zones = search_opts.get('extra_specs', {}).pop(
        'availability_zones', None)
    extra_specs = search_opts.pop('extra_specs', {})

    if extra_specs:
        search_vars['extra_specs'] = extra_specs

    if availability_zones:
        search_vars['availability_zones'] = availability_zones.split(',')

    if search_opts:
        # No other search options are currently supported
        return {}
    elif not search_vars:
        return share_types

    LOG.debug("Searching by: %s", search_vars)

    def _check_extra_specs_match(share_type, searchdict):
        for k, v in searchdict.items():
            if (k not in share_type['extra_specs'].keys()
                    or share_type['extra_specs'][k] != v):
                return False
            return True

    def _check_availability_zones_match(share_type, availability_zones):
        type_azs = share_type['extra_specs'].get('availability_zones')
        if type_azs:
            type_azs = type_azs.split(',')
            return set(availability_zones).issubset(set(type_azs))
        return True

    # search_option to filter_name mapping.
    filter_mapping = {
        'extra_specs': _check_extra_specs_match,
        'availability_zones': _check_availability_zones_match,
    }

    result = {}
    for type_name, type_args in share_types.items():
        # go over all filters in the list (*AND* operation)
        type_matches = True
        for opt, value in search_vars.items():
            try:
                filter_func = filter_mapping[opt]
            except KeyError:
                # no such filter - ignore it, go to next filter
                continue
            else:
                if not filter_func(type_args, value):
                    type_matches = False
                    break
        if type_matches:
            result[type_name] = type_args
    return result


def get_share_type(ctxt, id, expected_fields=None):
    """Retrieves single share type by id."""
    if id is None:
        msg = _("id cannot be None")
        raise exception.InvalidShareType(reason=msg)

    if ctxt is None:
        ctxt = context.get_admin_context()

    return db.share_type_get(ctxt, id, expected_fields=expected_fields)


def get_share_type_by_name(context, name):
    """Retrieves single share type by name."""
    if name is None:
        msg = _("name cannot be None")
        raise exception.InvalidShareType(reason=msg)

    return db.share_type_get_by_name(context, name)


def get_share_type_by_name_or_id(context, share_type=None):
    if not share_type:
        share_type_ref = get_default_share_type(context)
        if not share_type_ref:
            msg = _("Default share type not found.")
            raise exception.ShareTypeNotFound(message=msg)
        return share_type_ref

    if uuidutils.is_uuid_like(share_type):
        return get_share_type(context, share_type)
    else:
        return get_share_type_by_name(context, share_type)


def get_default_share_type(ctxt=None):
    """Get the default share type."""
    name = CONF.default_share_type

    if name is None:
        return {}

    if ctxt is None:
        ctxt = context.get_admin_context()

    share_type = {}
    try:
        share_type = get_share_type_by_name(ctxt, name)
        required_extra_specs = get_valid_required_extra_specs(
            share_type['extra_specs'])
        share_type['required_extra_specs'] = required_extra_specs
        return share_type
    except exception.ShareTypeNotFoundByName as e:
        # Couldn't find share type with the name in default_share_type
        # flag, record this issue and move on
        # TODO(zhiteng) consider add notification to warn admin
        LOG.exception('Default share type is not found, '
                      'please check default_share_type config: %s', e)
    except exception.InvalidExtraSpec as ex:
        LOG.exception('Default share type has invalid required extra'
                      ' specs: %s', ex)


def get_share_type_extra_specs(share_type_id, key=False):
    share_type = get_share_type(context.get_admin_context(),
                                share_type_id)
    extra_specs = share_type['extra_specs']
    if key:
        if extra_specs.get(key):
            return extra_specs.get(key)
        else:
            return False
    else:
        return extra_specs


def get_required_extra_specs():
    return constants.ExtraSpecs.REQUIRED


def get_optional_extra_specs():
    return constants.ExtraSpecs.OPTIONAL


def get_tenant_visible_extra_specs():
    return constants.ExtraSpecs.TENANT_VISIBLE


def get_boolean_extra_specs():
    return constants.ExtraSpecs.BOOLEAN


def is_valid_required_extra_spec(key, value):
    """Validates required extra_spec value.

    :param key: extra_spec name
    :param value: extra_spec value
    :return: None if provided extra_spec is not required
             True/False if extra_spec is required and valid or not.
    """
    if key not in get_required_extra_specs():
        return

    if key == constants.ExtraSpecs.DRIVER_HANDLES_SHARE_SERVERS:
        return strutils.bool_from_string(value, default=None) is not None

    return False


def get_valid_required_extra_specs(extra_specs):
    """Validates and returns required extra specs from dict.

    Raises InvalidExtraSpec if extra specs are not valid, or if any required
    extra specs are missing.
    """
    extra_specs = extra_specs or {}

    missed_extra_specs = set(get_required_extra_specs()) - set(extra_specs)

    if missed_extra_specs:
        specs = ",".join(missed_extra_specs)
        msg = _("Required extra specs '%s' not specified.") % specs
        raise exception.InvalidExtraSpec(reason=msg)

    required_extra_specs = {}

    for k in get_required_extra_specs():
        value = extra_specs.get(k, '')
        if not is_valid_required_extra_spec(k, value):
            msg = _("Value of required extra_spec %s is not valid") % k
            raise exception.InvalidExtraSpec(reason=msg)

        required_extra_specs[k] = value

    return required_extra_specs


def is_valid_csv(extra_spec_value):
    if not isinstance(extra_spec_value, six.string_types):
        extra_spec_value = six.text_type(extra_spec_value)
    values = extra_spec_value.split(',')
    return all([v.strip() for v in values])


def sanitize_csv(csv_string):
    return ','.join(value.strip() for value in csv_string.split(',')
                    if (csv_string and value))


def is_valid_optional_extra_spec(key, value):
    """Validates optional but standardized extra_spec value.

    :param key: extra_spec name
    :param value: extra_spec value
    :return: None if provided extra_spec is not required
             True/False if extra_spec is required and valid or not.
    """
    if key not in get_optional_extra_specs():
        return

    if key == constants.ExtraSpecs.SNAPSHOT_SUPPORT:
        return parse_boolean_extra_spec(key, value) is not None
    elif key == constants.ExtraSpecs.CREATE_SHARE_FROM_SNAPSHOT_SUPPORT:
        return parse_boolean_extra_spec(key, value) is not None
    elif key == constants.ExtraSpecs.REVERT_TO_SNAPSHOT_SUPPORT:
        return parse_boolean_extra_spec(key, value) is not None
    elif key == constants.ExtraSpecs.REPLICATION_TYPE_SPEC:
        return value in constants.ExtraSpecs.REPLICATION_TYPES
    elif key == constants.ExtraSpecs.MOUNT_SNAPSHOT_SUPPORT:
        return parse_boolean_extra_spec(key, value) is not None
    elif key == constants.ExtraSpecs.AVAILABILITY_ZONES:
        return is_valid_csv(value)
    elif key in [constants.ExtraSpecs.PROVISIONING_MAX_SHARE_SIZE,
                 constants.ExtraSpecs.PROVISIONING_MIN_SHARE_SIZE]:
        try:
            common.validate_integer(value, 'share_size', min_value=1)
            return True
        except ValueError:
            return False

    return False


def get_valid_optional_extra_specs(extra_specs):
    """Validates and returns optional/standard extra specs from dict.

    Raises InvalidExtraSpec if extra specs are not valid.
    """

    extra_specs = extra_specs or {}
    present_optional_extra_spec_keys = set(extra_specs).intersection(
        set(get_optional_extra_specs()))

    optional_extra_specs = {}

    for key in present_optional_extra_spec_keys:
        value = extra_specs.get(key, '')
        if not is_valid_optional_extra_spec(key, value):
            msg = _("Value of optional extra_spec %s is not valid.") % key
            raise exception.InvalidExtraSpec(reason=msg)

        optional_extra_specs[key] = value

    return optional_extra_specs


def add_share_type_access(context, share_type_id, project_id):
    """Add access to share type for project_id."""
    if share_type_id is None:
        msg = _("share_type_id cannot be None")
        raise exception.InvalidShareType(reason=msg)
    return db.share_type_access_add(context, share_type_id, project_id)


def remove_share_type_access(context, share_type_id, project_id):
    """Remove access to share type for project_id."""
    if share_type_id is None:
        msg = _("share_type_id cannot be None")
        raise exception.InvalidShareType(reason=msg)
    return db.share_type_access_remove(context, share_type_id, project_id)


def get_extra_specs_from_share(share):
    type_id = share.get('share_type_id', None)
    return get_share_type_extra_specs(type_id)


def parse_boolean_extra_spec(extra_spec_key, extra_spec_value):
    """Parse extra spec values of the form '<is> True' or '<is> False'

    This method returns the boolean value of an extra spec value.  If
    the value does not conform to the standard boolean pattern, it raises
    an InvalidExtraSpec exception.
    """
    if not isinstance(extra_spec_value, six.string_types):
        extra_spec_value = six.text_type(extra_spec_value)

    match = re.match(r'^<is>\s*(?P<value>True|False)$',
                     extra_spec_value.strip(),
                     re.IGNORECASE)
    if match:
        extra_spec_value = match.group('value')
    try:
        return strutils.bool_from_string(extra_spec_value, strict=True)
    except ValueError:
        msg = (_('Invalid boolean extra spec %(key)s : %(value)s') %
               {'key': extra_spec_key, 'value': extra_spec_value})
        raise exception.InvalidExtraSpec(reason=msg)


def provision_filter_on_size(context, share_type, size):
    """This function filters share provisioning requests on size limits.

    If a share type has provisioning size min/max set, this filter
    will ensure that the share size requested is within the size
    limits specified in the share type.
    """
    if not share_type:
        share_type = get_default_share_type()
    if share_type:
        size_int = int(size)
        extra_specs = share_type.get('extra_specs', {})
        min_size = extra_specs.get(MIN_SIZE_KEY)
        if min_size and size_int < int(min_size):
            msg = _("Specified share size of '%(req_size)d' is less "
                    "than the minimum required size of '%(min_size)s' "
                    "for share type '%(sha_type)s'."
                    ) % {'req_size': size_int, 'min_size': min_size,
                         'sha_type': share_type['name']}
            raise exception.InvalidInput(reason=msg)
        max_size = extra_specs.get(MAX_SIZE_KEY)
        if max_size and size_int > int(max_size):
            msg = _("Specified share size of '%(req_size)d' is "
                    "greater than the maximum allowable size of "
                    "'%(max_size)s' for share type '%(sha_type)s'."
                    ) % {'req_size': size_int, 'max_size': max_size,
                         'sha_type': share_type['name']}
            raise exception.InvalidInput(reason=msg)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""add access level

Revision ID: 211836bf835c
Revises: 162a3e673105
Create Date: 2014-12-19 05:34:06.790159

"""

# revision identifiers, used by Alembic.
revision = '211836bf835c'
down_revision = '162a3e673105'

from alembic import op
import sqlalchemy as sa

from manila.common import constants


def upgrade():
    op.add_column('share_access_map',
                  sa.Column('access_level', sa.String(2),
                            default=constants.ACCESS_LEVEL_RW))


def downgrade():
    op.drop_column('share_access_map', 'access_level')



# Copyright (c) 2017 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import versionutils
from oslo_policy import policy

RULE_ADMIN_OR_OWNER = 'rule:admin_or_owner'
RULE_ADMIN_API = 'rule:admin_api'
RULE_DEFAULT = 'rule:default'

deprecation_msg = ("The `context_is_admin` check is superseded by more "
                   "specific check strings that consume system and project "
                   "scope attributes from keystone tokens.")
DEPRECATED_CONTEXT_IS_ADMIN = policy.DeprecatedRule(
    name='context_is_admin',
    check_str='role:admin',
    deprecated_reason=deprecation_msg,
    deprecated_since=versionutils.deprecated.WALLABY
)

# Generic policy check string for system administrators. These are the people
# who need the highest level of authorization to operate the deployment.
# They're allowed to create, read, update, or delete any system-specific
# resource. They can also operate on project-specific resources where
# applicable (e.g., cleaning up shares or snapshots).
SYSTEM_ADMIN = 'rule:system-admin'

# Generic policy check string for system users who don't require all the
# authorization that system administrators typically have. This persona, or
# check string, typically isn't used by default, but it's existence it useful
# in the event a deployment wants to offload some administrative action from
# system administrator to system members.
SYSTEM_MEMBER = 'rule:system-member'

# Generic policy check string for read-only access to system-level resources.
# This persona is useful for someone who needs access for auditing or even
# support. These uses are also able to view project-specific resources where
# applicable (e.g., listing all shares in the deployment, regardless of the
# project they belong to).
SYSTEM_READER = 'rule:system-reader'

# This check string is reserved for actions that require the highest level of
# authorization on a project or resources within the project (e.g., resyncing a
# share replica).
PROJECT_ADMIN = 'rule:project-admin'

# This check string is the primary use case for typical end-users, who are
# working with resources that belong to a project (e.g., managing shares or
# share replicas).
PROJECT_MEMBER = 'rule:project-member'

# This check string should only be used to protect read-only project-specific
# resources. It should not be used to protect APIs that make writable changes
# (e.g., updating a share or snapshot).
PROJECT_READER = 'rule:project-reader'

# The following are common composite check strings that are useful for
# protecting APIs designed to operate with multiple scopes (e.g., a system
# administrator should be able to delete any share in the deployment, a
# project member should only be able to delete shares in their project).
SYSTEM_ADMIN_OR_PROJECT_ADMIN = (
    '(' + SYSTEM_ADMIN + ') or (' + PROJECT_ADMIN + ')'
)
SYSTEM_ADMIN_OR_PROJECT_MEMBER = (
    '(' + SYSTEM_ADMIN + ') or (' + PROJECT_MEMBER + ')'
)
SYSTEM_OR_PROJECT_READER = (
    '(' + SYSTEM_READER + ') or (' + PROJECT_READER + ')'
)

rules = [
    # ***Default OpenStack scoped personas*** #
    policy.RuleDefault(
        name='system-admin',
        check_str='role:admin and '
                  'system_scope:all',
        description='System scoped Administrator',
        scope_types=['system']),
    policy.RuleDefault(
        name='system-member',
        check_str='role:member and '
                  'system_scope:all',
        description='System scoped Member',
        scope_types=['system']),
    policy.RuleDefault(
        name='system-reader',
        check_str='role:reader and '
                  'system_scope:all',
        description='System scoped Reader',
        scope_types=['system']),
    policy.RuleDefault(
        name='project-admin',
        check_str='role:admin and '
                  'project_id:%(project_id)s',
        description='Project scoped Administrator',
        scope_types=['project']),
    policy.RuleDefault(
        name='project-member',
        check_str='role:member and '
                  'project_id:%(project_id)s',
        description='Project scoped Member',
        scope_types=['project']),
    policy.RuleDefault(
        name='project-reader',
        check_str='role:reader and '
                  'project_id:%(project_id)s',
        description='Project scoped Reader',
        scope_types=['project']),

    # ***Special personas for Manila*** #
    policy.RuleDefault(
        name='context_is_admin',
        check_str='rule:system-admin',
        description='Privileged users checked via "context.is_admin"',
        deprecated_rule=DEPRECATED_CONTEXT_IS_ADMIN,
        scope_types=['system']),

    # ***Legacy/deprecated unscoped rules*** #
    # can be removed after "enforce_scope" defaults to True in oslo.policy
    policy.RuleDefault(
        name='admin_or_owner',
        check_str='is_admin:True or project_id:%(project_id)s',
        description='Administrator or Member of the project'),
    policy.RuleDefault(
        name='default',
        check_str=RULE_ADMIN_OR_OWNER,
        description='Default rule for most non-Admin APIs'),
    policy.RuleDefault(
        name='admin_api',
        check_str='is_admin:True',
        description='Default rule for most Admin APIs.'),
]


def list_rules():
    return rules



# Copyright 2014 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manila.api import common


class ViewBuilder(common.ViewBuilder):
    """Model a server API response as a python dictionary."""

    _collection_name = 'share_networks'
    _detail_version_modifiers = ["add_gateway", "add_mtu", "add_nova_net_id",
                                 "add_subnets",
                                 "add_status_and_sec_service_update_fields"]

    def build_share_network(self, request, share_network):
        """View of a share network."""

        return {'share_network': self._build_share_network_view(
            request, share_network)}

    def build_share_networks(self, request, share_networks, is_detail=True):
        return {'share_networks':
                [self._build_share_network_view(
                    request, share_network, is_detail)
                 for share_network in share_networks]}

    def build_security_service_update_check(self, request, params, result):
        """View of security service add or update check."""
        context = request.environ['manila.context']
        requested_operation = {
            'operation': ('update_security_service'
                          if params.get('current_service_id')
                          else 'add_security_service'),
            'current_security_service': params.get('current_service_id'),
            'new_security_service': (params.get('new_service_id') or
                                     params.get('security_service_id'))
        }
        view = {
            'compatible': result['compatible'],
            'requested_operation': requested_operation,
        }
        if context.is_admin:
            view['hosts_check_result'] = result['hosts_check_result']
        return view

    def _update_share_network_info(self, request, share_network):
        for sns in share_network.get('share_network_subnets') or []:
            if sns.get('is_default') and sns.get('is_default') is True:
                share_network.update({
                    'neutron_net_id': sns.get('neutron_net_id'),
                    'neutron_subnet_id': sns.get('neutron_subnet_id'),
                    'network_type': sns.get('network_type'),
                    'segmentation_id': sns.get('segmentation_id'),
                    'cidr': sns.get('cidr'),
                    'ip_version': sns.get('ip_version'),
                    'gateway': sns.get('gateway'),
                    'mtu': sns.get('mtu'),
                })

    def _build_share_network_view(self, request, share_network,
                                  is_detail=True):
        sn = {
            'id': share_network.get('id'),
            'name': share_network.get('name'),
        }
        if is_detail:
            self._update_share_network_info(request, share_network)
            sn.update({
                'project_id': share_network.get('project_id'),
                'created_at': share_network.get('created_at'),
                'updated_at': share_network.get('updated_at'),
                'neutron_net_id': share_network.get('neutron_net_id'),
                'neutron_subnet_id': share_network.get('neutron_subnet_id'),
                'network_type': share_network.get('network_type'),
                'segmentation_id': share_network.get('segmentation_id'),
                'cidr': share_network.get('cidr'),
                'ip_version': share_network.get('ip_version'),
                'description': share_network.get('description'),
            })

            self.update_versioned_resource_dict(request, sn, share_network)
        return sn

    @common.ViewBuilder.versioned_method("2.51")
    def add_subnets(self, context, network_dict, network):
        subnets = [{
            'id': sns.get('id'),
            'availability_zone': sns.get('availability_zone'),
            'created_at': sns.get('created_at'),
            'updated_at': sns.get('updated_at'),
            'segmentation_id': sns.get('segmentation_id'),
            'neutron_net_id': sns.get('neutron_net_id'),
            'neutron_subnet_id': sns.get('neutron_subnet_id'),
            'ip_version': sns.get('ip_version'),
            'cidr': sns.get('cidr'),
            'network_type': sns.get('network_type'),
            'mtu': sns.get('mtu'),
            'gateway': sns.get('gateway'),
        } for sns in network.get('share_network_subnets')]

        network_dict['share_network_subnets'] = subnets
        attr_to_remove = [
            'neutron_net_id', 'neutron_subnet_id', 'network_type',
            'segmentation_id', 'cidr', 'ip_version', 'gateway', 'mtu']
        for attr in attr_to_remove:
            network_dict.pop(attr)

    @common.ViewBuilder.versioned_method("2.18")
    def add_gateway(self, context, network_dict, network):
        network_dict['gateway'] = network.get('gateway')

    @common.ViewBuilder.versioned_method("2.20")
    def add_mtu(self, context, network_dict, network):
        network_dict['mtu'] = network.get('mtu')

    @common.ViewBuilder.versioned_method("1.0", "2.25")
    def add_nova_net_id(self, context, network_dict, network):
        network_dict['nova_net_id'] = None

    @common.ViewBuilder.versioned_method("2.63")
    def add_status_and_sec_service_update_fields(
            self, context, network_dict, network):
        network_dict['status'] = network.get('status')
        network_dict['security_service_update_support'] = network.get(
            'security_service_update_support')



# Copyright 2012 OpenStack LLC.
# Copyright (c) 2015 Mirantis inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import webob

from manila.api.openstack import wsgi
from manila.api.views import quota_class_sets as quota_class_sets_views
from manila import db
from manila import exception
from manila import quota

QUOTAS = quota.QUOTAS


class QuotaClassSetsMixin(object):
    """The Quota Class Sets API controller common logic.

    Mixin class that should be inherited by Quota Class Sets API controllers,
    which are used for different API URLs and microversions.
    """

    resource_name = "quota_class_set"
    _view_builder_class = quota_class_sets_views.ViewBuilder

    @wsgi.Controller.authorize("show")
    def _show(self, req, id):
        context = req.environ['manila.context']
        try:
            db.authorize_quota_class_context(context, id)
        except exception.NotAuthorized:
            raise webob.exc.HTTPForbidden()

        return self._view_builder.detail_list(
            req, QUOTAS.get_class_quotas(context, id), id)

    @wsgi.Controller.authorize("update")
    def _update(self, req, id, body):
        context = req.environ['manila.context']
        quota_class = id
        for key in body.get(self.resource_name, {}).keys():
            if key in QUOTAS:
                value = int(body[self.resource_name][key])
                try:
                    db.quota_class_update(context, quota_class, key, value)
                except exception.QuotaClassNotFound:
                    db.quota_class_create(context, quota_class, key, value)
                except exception.AdminRequired:
                    raise webob.exc.HTTPForbidden()
        return self._view_builder.detail_list(
            req, QUOTAS.get_class_quotas(context, quota_class))


class QuotaClassSetsControllerLegacy(QuotaClassSetsMixin, wsgi.Controller):
    """Deprecated Quota Class Sets API controller.

    Used by legacy API v1 and v2 microversions from 2.0 to 2.6.
    Registered under deprecated API URL 'os-quota-class-sets'.
    """

    @wsgi.Controller.api_version('1.0', '2.6')
    def show(self, req, id):
        return self._show(req, id)

    @wsgi.Controller.api_version('1.0', '2.6')
    def update(self, req, id, body):
        return self._update(req, id, body)


class QuotaClassSetsController(QuotaClassSetsMixin, wsgi.Controller):
    """Quota Class Sets API controller.

    Used only by API v2 starting from microversion 2.7.
    Registered under API URL 'quota-class-sets'.
    """

    @wsgi.Controller.api_version('2.7')
    def show(self, req, id):
        return self._show(req, id)

    @wsgi.Controller.api_version('2.7')
    def update(self, req, id, body):
        return self._update(req, id, body)


def create_resource_legacy():
    return wsgi.Resource(QuotaClassSetsControllerLegacy())


def create_resource():
    return wsgi.Resource(QuotaClassSetsController())
